package com.jkcredit.invoice.consumer;

import com.jkcredit.invoice.common.CommonConstants;
import com.jkcredit.invoice.model.entity.User;
import com.jkcredit.invoice.service.UserService;
import com.rabbitmq.client.Channel;
import lombok.extern.slf4j.Slf4j;
import org.springframework.amqp.core.Message;
import org.springframework.amqp.rabbit.annotation.*;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.messaging.handler.annotation.Payload;
import org.springframework.stereotype.Component;

import java.io.IOException;

/**
 * @description:
 * @author: sunzhaoning
 * @create: 2019-06-20 09:04
 * @version: V1.0
 **/
@Slf4j
@Component
public class B2bChargeConsumer {

    @Autowired
    private UserService userService;

    @RabbitHandler
    @RabbitListener(bindings = @QueueBinding(
            exchange = @Exchange(
                    name = CommonConstants.B2B_CHARGE_EXCHANGE,
                    type = "topic",
                    ignoreDeclarationExceptions = "true"
            ),
            value = @Queue(
                    value = CommonConstants.B2B_CHARGE_QUEUE
            ),
            key = "b2b_charge.*"
    ))
    public void process(@Payload User user, Channel channel, Message message) throws IOException {
        try {
            synchronized (this) {
                user = userService.getById(user.getId());
               // user.setMoney(user.getMoney() - user.getPrice());
                //更新余额
                //userService.charge(user);
            }
            //告诉服务器收到这条消息 已经被我消费了 可以在队列删掉 这样以后就不会再发了 否则消息服务器以为这条消息没处理掉 后续还会在发
            channel.basicAck(message.getMessageProperties().getDeliveryTag(),false);
        } catch (IOException e) {
            e.printStackTrace();
            //丢弃这条消息
            //channel.basicNack(message.getMessageProperties().getDeliveryTag(), false,false);
            //ack返回false，并重新回到队列
            channel.basicNack(message.getMessageProperties().getDeliveryTag(), false, true);
            log.error("接受失败");
        }

    }

}
