package com.jkcredit.invoice.controller;

import cn.afterturn.easypoi.excel.entity.ExportParams;
import com.alibaba.fastjson.JSON;
import com.alibaba.fastjson.JSONArray;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.jkcredit.invoice.annotation.LoginRequired;
import com.jkcredit.invoice.common.CommonConstants;
import com.jkcredit.invoice.model.entity.CarFreeCarrierBillEnd;
import com.jkcredit.invoice.model.entity.CarFreeCarrierBillStart;
import com.jkcredit.invoice.model.vo.CarFreeCarrierBillVo;
import com.jkcredit.invoice.service.CarFreeCarrierBillEndService;
import com.jkcredit.invoice.service.CarFreeCarrierBillService;
import com.jkcredit.invoice.service.CarFreeCarrierBillStartService;
import com.jkcredit.invoice.util.DownExcelTemplateUtil;
import com.jkcredit.invoice.util.ExportUtil;
import com.jkcredit.invoice.util.ReadExcelUtil;
import com.jkcredit.invoice.util.RespR;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.*;
import org.springframework.web.multipart.MultipartFile;

import javax.servlet.http.HttpServletResponse;
import java.io.IOException;
import java.util.ArrayList;
import java.util.List;

/**
 * @description: 运单控制器
 * @author: sunzhaoning
 * @create: 2019-05-31 10:29
 * @version: V1.0
 **/
@Api(tags = "运单管理")
@RestController
@Slf4j
@RequestMapping("/carFreeCarrierBill")
public class CarFreeCarrierBillController {

    @Autowired
    private CarFreeCarrierBillService carFreeCarrierBillService;

    @Autowired
    private CarFreeCarrierBillStartService carFreeCarrierBillStartService;

    @Autowired
    private CarFreeCarrierBillEndService carFreeCarrierBillEndService;

    @LoginRequired
    @PostMapping("/bill")
    @ApiOperation(value = "批量导入运单信息", notes = "批量导入运单信息")
    public RespR<Boolean> billImport(@RequestParam("file") MultipartFile file, @RequestParam Integer userId) {
        List<CarFreeCarrierBillVo> list = ReadExcelUtil.readExcel(file, CarFreeCarrierBillVo.class);
        log.info("批量导入实时运单传入参数：{}", JSON.toJSONString(list));
        return carFreeCarrierBillService.addCarFreeCarrierBillList(list, userId);
    }

    @LoginRequired
    @PostMapping("/historyBill")
    @ApiOperation(value = "批量导入历史运单信息", notes = "批量导入历史运单信息")
    public RespR<Boolean> historyBillImport(@RequestParam("file") MultipartFile file, @RequestParam Integer userId) {
        List<CarFreeCarrierBillVo> list = ReadExcelUtil.readExcel(file, CarFreeCarrierBillVo.class);
        log.info("批量导入历史运单传入参数：{}", JSON.toJSONString(list));
        return carFreeCarrierBillService.addCarFreeCarrierBillHistoryList(list, userId);
    }

    @ApiOperation(value = "下载运单模板", notes = "下载运单模板")
    @GetMapping("/downTemp")
    public void donwTemp(HttpServletResponse response) {
        DownExcelTemplateUtil.downloadExcel(response, "运单模板.xlsx");
    }

    @PostMapping("/list")
    @ApiOperation(value = "查询运单列表", notes = "查询运单列表")
    @LoginRequired
    public RespR getCarFreeCarrierRegisterList(Page page, CarFreeCarrierBillStart carFreeCarrierBillStart) {
        return new RespR<>(carFreeCarrierBillStartService.getCarFreeCarrierBillStartPage(page, carFreeCarrierBillStart));
    }

    @ApiOperation(value = "下载运单结束模板", notes = "下载运单结束模板")
    @GetMapping("/downEndTemp")
    public void donwTEndemp(HttpServletResponse response) {
        DownExcelTemplateUtil.downloadExcel(response, "运单结束模板.xlsx");
    }

    @LoginRequired
    @PostMapping("/billEnd")
    @ApiOperation(value = "批量导入运单结束信息", notes = "批量导入运单结束信息")
    public RespR<Boolean> billEndImport(@RequestParam("file") MultipartFile file, @RequestParam Integer userId) {
        List<CarFreeCarrierBillEnd> list = ReadExcelUtil.readExcel(file, CarFreeCarrierBillEnd.class);
//        List<CarFreeCarrierBillEnd> list = JSONArray.parseArray("[{\"endTime\":\"Tue Feb 12 12:21:21 CST 2019\",\"invoiceTime\":1,\"num\":\"123123\",\"realDestAddr\":\"beij\"},{\"endTime\":\"Tue Feb 12 12:21:21 CST 2019\",\"invoiceTime\":1,\"num\":\"123123\",\"realDestAddr\":\"beij\"}]",CarFreeCarrierBillEnd.class);
        log.info("批量导入结束运单传入参数：{}", JSON.toJSONString(list));
        return carFreeCarrierBillService.addCarFreeCarrierBillEndList(list, userId);
    }

    @PostMapping("/endList")
    @ApiOperation(value = "查询运单结束列表", notes = "查询运单结束列表")
    @LoginRequired
    public RespR getCarFreeCarrierEndList(Page page, CarFreeCarrierBillEnd carFreeCarrierBillEnd) {
        return new RespR<>(carFreeCarrierBillEndService.getCarFreeCarrierBillEndPage(page, carFreeCarrierBillEnd));
    }

    @ApiOperation(value = "导出运单列表", notes = "导出运单列表")
    @GetMapping("/billExport")
    public void export(Integer userId, HttpServletResponse response) throws IOException {
        List<CarFreeCarrierBillStart> list = carFreeCarrierBillStartService.getCarFreeCarrierBillSuccessList(userId, CommonConstants.SUCCESS);
//        List<CarFreeCarrierBillStart> newList = new ArrayList<>();

        ExportUtil.exportExcel(list, CarFreeCarrierBillStart.class, "运单列表", new ExportParams("运单列表", "运单列表"), response);
    }

    @PostMapping("/notFinish")
    @ApiOperation(value = "查询未开票完成运单列表", notes = "查询未开票完成运单列表")
    @LoginRequired
    public RespR getNotFinishListPage(Page page, CarFreeCarrierBillStart carFreeCarrierBillStart) {
        return new RespR<>(carFreeCarrierBillStartService.getNotFinishListPage(page, carFreeCarrierBillStart));
    }

    @PostMapping("/finish")
    @ApiOperation(value = "查询开票完成运单列表", notes = "查询开票完成运单列表")
    @LoginRequired
    public RespR getFinishListPage(Page page, CarFreeCarrierBillStart carFreeCarrierBillStart, Long startTime, Long endTime) {
        if (endTime != null) {
            carFreeCarrierBillStart.setCreateStartTime(startTime.toString());
        }
        if (endTime != null) {
            carFreeCarrierBillStart.setCreateEndTime(endTime.toString());
        }
        return new RespR<>(carFreeCarrierBillStartService.getFinishListPage(page, carFreeCarrierBillStart));
    }


}
