package com.jkcredit.invoice.controller.Customer;

import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.jkcredit.invoice.annotation.LoginRequired;
import com.jkcredit.invoice.model.entity.Customer;
import com.jkcredit.invoice.model.entity.CustomerRec;
import com.jkcredit.invoice.model.entity.CustomerRecharge;
import com.jkcredit.invoice.model.entity.User;
import com.jkcredit.invoice.service.CustomerService;
import com.jkcredit.invoice.util.DateUtil;
import com.jkcredit.invoice.util.RespR;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;

/**
 * 客户操作
 */
@Api(tags = "客户操作")
@RestController
@RequestMapping(value = {"/customer"})
public class CustomerController {
    @Autowired
    CustomerService customerService;
    /**
     * 分页查询客户
     *
     * @param page    参数集
     * @return 用户集合
     */
    @PostMapping("/findCustomer")
    @ApiOperation(value="分页查询客户", notes="分页查询客户")
    @LoginRequired
    public RespR getCustomersByPage(Page page, Customer customer) {

        return new RespR(customerService.findAllCustomers(page, customer));
    }
    /**
     * 添加客户
     * @return 用户集合
     */
    @PostMapping("/addCustomer")
    @ApiOperation(value="添加客户", notes="添加客户")
    @LoginRequired
    public RespR addCustomer(@RequestBody  Customer customer) {
        return new RespR(customerService.addCustomers(customer),"客户添加失败，请查看是否重复");
    }
    /**
     * 修改客户状态
     * @return 用户集合
     */
    @PostMapping("/updateCustomer")
    @ApiOperation(value="更新用户", notes="更新用户")
    @LoginRequired
    public RespR updateCustomer(@RequestBody  Customer customer) {

        return new RespR(customerService.updateCustomer(customer));
    }
    /**
     * 充值
     * @return 用户集合
     */
    @PostMapping("/customRecharge")
    @ApiOperation(value="账号充值", notes="账号充值")
    @LoginRequired
    public RespR customRecharge(@RequestBody CustomerRecharge customerRecharge) {

        return new RespR(customerService.customRecharge(customerRecharge));
    }
}
