package com.jkcredit.invoice.controller;

import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.jkcredit.invoice.annotation.LoginRequired;
import com.jkcredit.invoice.model.entity.Invoice;
import com.jkcredit.invoice.service.InvoiceService;
import com.jkcredit.invoice.util.RespR;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.*;

/**
 * @description: 发票信息控制器
 * @author: sunzhaoning
 * @create: 2019-05-30 09:18
 * @version: V1.0
 **/
@Api(tags = "发票管理")
@RestController
@RequestMapping("/invoice")
@Slf4j
public class InvoiceController {

    @Autowired
    private InvoiceService invoiceService;

    /**
     * 新增发票信息
     * @param invoice
     * @return
     */
    @PostMapping("/add")
    @ApiOperation(value = "新增发票信息", notes = "新增发票信息")
    @LoginRequired
    public RespR addInvoice(@RequestBody Invoice invoice) {
        return invoiceService.saveInvoice(invoice);
    }

    /**
     * 查询发票列表
     * @param page
     * @return
     */
    @PostMapping("/list")
    @ApiOperation(value = "查询发票列表", notes = "查询发票列表")
    @LoginRequired
    public RespR getInvoiceList(Page page,Invoice invoice){
        return new RespR<>(invoiceService.getInvoicesPage(page,invoice));
    }

    /**
     * 根据用户查询发票列表
     * @param userId
     * @return
     */
    @PostMapping("/{userId}")
    @ApiOperation(value = "根据用户id查询发票信息", notes = "根据用户id查询发票信息")
    @LoginRequired
    public RespR getInvoiceListByUserId(@PathVariable Integer userId){
        return new RespR(invoiceService.getByUserId(userId));
    }

}
