package com.jkcredit.invoice.controller;

import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.jkcredit.invoice.annotation.LoginRequired;
import com.jkcredit.invoice.model.entity.SearchInvoiceData;
import com.jkcredit.invoice.model.entity.SearchInvoiceResult;
import com.jkcredit.invoice.service.SearchInvoiceDataService;
import com.jkcredit.invoice.service.SearchInvoiceResultService;
import com.jkcredit.invoice.util.RespR;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import org.apache.poi.ss.usermodel.Workbook;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;

import javax.servlet.http.HttpServletResponse;
import java.io.IOException;
import java.net.URLEncoder;
import java.util.ArrayList;
import java.util.List;

@RestController
@RequestMapping("/searchInvoiceResult")
@Api(tags = "查询发票数据")
public class SearchInvoiceResultController {
    @Autowired
    private SearchInvoiceResultService searchInvoiceResultService;
    @Autowired
    private SearchInvoiceDataService searchInvoiceDataService;
    /**
     * 查询发票数据列表
     *
     * @param page
     * @return
     */
    @PostMapping("/list")
    @ApiOperation(value = "查询发票数据列表", notes = "查询发票数据列表")
    @LoginRequired
    public RespR getSearchResultList(Page page, SearchInvoiceResult searchInvoiceResult,String batchNumber) {
        return new RespR<>(searchInvoiceResultService.getSearchInvoiceResultPage(page, searchInvoiceResult,batchNumber));
    }

    /**
     * 查询发票数据列表(表num_invoice与search_invoice_result数据)
     *
     * @param page
     * @return
     */
    @PostMapping("/invoiceDataList")
    @ApiOperation(value = "查询发票数据列表", notes = "查询发票数据列表")
    @LoginRequired
    public RespR getSearchInvoiceDataList(Page page, SearchInvoiceData searchInvoiceData) {
        return new RespR(searchInvoiceDataService.getInvoiceDataPage(page, searchInvoiceData));
    }

    /**
     * 导出发票列表
     * @param userId
     * @param waybillNum
     * @param response
     * @param batchNumber
     * @param startTime
     * @param endTime
     * @throws IOException
     */
    @ApiOperation(value = "导出发票列表",notes = "导出发票列表")
    @GetMapping("/export")
    public void export(Integer userId,String waybillNum, HttpServletResponse response,String batchNumber,Long startTime,Long endTime) throws IOException {
        try {
            response.setCharacterEncoding("UTF-8");
            response.setHeader("content-Type", "application/vnd.ms-excel");
            response.setHeader("Content-Disposition", "attachment;filename=" + URLEncoder.encode("发票列表" + ".xlsx", "UTF-8"));
            Workbook workbook = searchInvoiceResultService.exportExcelToTemplate(userId, waybillNum, batchNumber, startTime, endTime);
            workbook.write(response.getOutputStream());
        } catch (Exception e) {
            throw new IOException(e.getMessage());
        }
    }

    /**
     * 查询发票统计
     * @param searchInvoiceResult
     * @return
     */
    @PostMapping("/statistics")
    @ApiOperation(value = "查询发票统计", notes = "查询发票统计")
    @LoginRequired
    public RespR statisticsData(SearchInvoiceResult searchInvoiceResult,String batchNumber) {
        List<SearchInvoiceResult> list = new ArrayList<>();
        list.add(searchInvoiceResultService.statisticsData(searchInvoiceResult,batchNumber));
        return new RespR<>(list);
    }

    @PostMapping("/dailyBill")
    @ApiOperation(value = "查询发票统计", notes = "查询发票统计")
    public RespR getDailyBilling(Page page,Integer userId) {
        return new RespR<>(searchInvoiceResultService.getDailyBilling(page,userId));
    }
}
