package com.jkcredit.invoice.service.impl;

import com.alibaba.fastjson.JSON;
import com.alibaba.fastjson.JSONArray;
import com.alibaba.fastjson.JSONObject;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.google.common.util.concurrent.ThreadFactoryBuilder;
import com.jkcredit.invoice.common.CommonConstants;
import com.jkcredit.invoice.mapper.B2bInvoiceApplyMapper;
import com.jkcredit.invoice.model.entity.B2bInvoiceApply;
import com.jkcredit.invoice.model.entity.CarEtcCard;
import com.jkcredit.invoice.model.entity.CardTrade;
import com.jkcredit.invoice.service.B2bInvoiceApplyService;
import com.jkcredit.invoice.service.CarEtcCardService;
import com.jkcredit.invoice.service.CardTradeService;
import com.jkcredit.invoice.util.DateUtil;
import com.jkcredit.invoice.util.HttpUtil;
import com.jkcredit.invoice.util.RespR;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.text.SimpleDateFormat;
import java.util.Date;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.concurrent.*;

/**
 * @description:
 * @author: sunzhaoning
 * @create: 2019-06-19 13:46
 * @version: V1.0
 **/
@Slf4j
@Service
public class B2bInvoiceApplyServiceImpl extends ServiceImpl<B2bInvoiceApplyMapper, B2bInvoiceApply> implements B2bInvoiceApplyService {

    @Autowired
    private CardTradeService cardTradeService;
    @Autowired
    private CarEtcCardService carEtcCardService;
    @Autowired
    private HttpUtil httpUtil;

    @Override
    public IPage getB2bInvoiceApplyPage(Page page, B2bInvoiceApply b2bInvoiceApply) {
        return baseMapper.getB2bInvoiceApplyPage(page, b2bInvoiceApply);
    }

    @Override
    public RespR<Boolean> invoiceApply(List<CardTrade> cardTradeList) {
        SimpleDateFormat sdf = new SimpleDateFormat("yyyy-MM-dd HH:mm:ss");
        //线程分割数
        int count = 20;
        //获取list大小
        int listSize = cardTradeList.size();
        //线程数
        int threadSize = (listSize / count) + 1;

        ThreadFactory namedThreadFactory = new ThreadFactoryBuilder().setNameFormat("thread-invoiceApply-%d").build();

        ExecutorService executor = new ThreadPoolExecutor(threadSize,threadSize,200L,TimeUnit.MILLISECONDS,new LinkedBlockingQueue<>(),namedThreadFactory);

        CountDownLatch countDownLatch = new CountDownLatch(threadSize);
        List<CardTrade> newList;
        for (int i = 0; i < threadSize; i++) {
            if ((i + 1) == threadSize) {
                int startIndex = (i * count);
                int endIndex = cardTradeList.size();
                newList = cardTradeList.subList(startIndex, endIndex);
            } else {
                int startIndex = i * count;
                ;
                int endIndex = (i + 1) * count;
                newList = cardTradeList.subList(startIndex, endIndex);
            }

            List<CardTrade> finalNewList = newList;
            executor.execute(() -> {
                if (null != finalNewList) {
                    finalNewList.stream().forEach(cardTrade -> {
                        //开始时间
                        String startTime = sdf.format(new Date(Long.parseLong(cardTrade.getStartExTime())));
                        //结束时间
                        String endTime = sdf.format(new Date(Long.parseLong(cardTrade.getEndExTime())));
                        //根据车牌号获取ETC卡号
                        List<CarEtcCard> carEtcCards = carEtcCardService.getCarEtcCardByPlateNum(cardTrade.getPlateNum());
                        for (CarEtcCard carEtcCard : carEtcCards) {
                            //设置ETC卡号
                            cardTrade.setCardId(carEtcCard.getCardId());

                            cardTrade.setStartExTime(startTime);
                            cardTrade.setEndExTime(endTime);

                            //根据企业编号、开始时间、结束时间、卡号查询交易id集合
                            List<CardTrade> tradeIdList = cardTradeService.getTradeIdList(cardTrade);
                            Map<String, Object> map = new HashMap<>(5);
                            map.put("companyNum", cardTrade.getCompanyNum());
                            map.put("cardId", cardTrade.getCardId());
                            map.put("tradeIdModel", tradeIdList);

                            String sendResult = httpUtil.getCheckResult(map, CommonConstants.B2B_INVOICE_APPLY_V1_API);
                            log.info("发票申请接口返回：{},请求参数：{}",sendResult,JSON.toJSONString(map));
                            //转换结果为jsonobject
                            JSONObject sendResultJson = JSON.parseObject(sendResult);
                            //获取返回data值
                            String dataCode = sendResultJson.getString("data");
                            //判断返回值是否为1 如果是1 设置状态为成功
                            if ("1".equals(dataCode)) {
                                //从返回值中获取返回信息
                                String msg = sendResultJson.getString("msg");
                                //将msg转换为json集合
                                JSONObject msgObject = JSON.parseObject(msg);
                                B2bInvoiceApply b2bInvoiceApply = null;
                                //申请 ID
                                String applyId = msgObject.getString("applyId");
                                //发票数量
                                Integer invoiceCount = msgObject.getInteger("invoiceCount");
                                //发票数量
                                Long invoiceAmount = msgObject.getLong("invoiceAmount");
                                //发票税额
                                Long invoiceTaxAmount = msgObject.getLong("invoiceTaxAmount");
                                //接受时间
                                String receiveTime = msgObject.getString("receiveTime");

                                b2bInvoiceApply.setApplyId(applyId);
                                b2bInvoiceApply.setCardId(cardTrade.getCardId());
                                b2bInvoiceApply.setInvoiceCount(invoiceCount);
                                b2bInvoiceApply.setInvoiceAmount(invoiceAmount);
                                b2bInvoiceApply.setInvoiceTaxAmount(invoiceTaxAmount);
                                b2bInvoiceApply.setReceiveTime(receiveTime);
                                b2bInvoiceApply.setCompanyNum(cardTrade.getCompanyNum());
                                b2bInvoiceApply.setPlateNum(cardTrade.getPlateNum());
                                b2bInvoiceApply.setCreateTime(System.currentTimeMillis());
                                //插入数据库
                                this.save(b2bInvoiceApply);
                            }
                        }

                    });
                }
                countDownLatch.countDown();
            });
        }
        try {
            countDownLatch.await();
        } catch (InterruptedException e) {
            e.printStackTrace();
        }

        executor.shutdown();
        return new RespR<>(Boolean.TRUE);
    }
}
