package com.jkcredit.invoice.service.impl;

import com.alibaba.fastjson.JSON;
import com.alibaba.fastjson.JSONArray;
import com.alibaba.fastjson.JSONObject;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.jkcredit.invoice.common.CommonConstants;
import com.jkcredit.invoice.mapper.B2bInvoicePackageMapper;
import com.jkcredit.invoice.model.entity.B2bInvoicePackage;
import com.jkcredit.invoice.model.entity.User;
import com.jkcredit.invoice.service.B2bInvoicePackageService;
import com.jkcredit.invoice.service.UserService;
import com.jkcredit.invoice.util.HttpUtil;
import com.jkcredit.invoice.util.RespR;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.text.SimpleDateFormat;
import java.util.*;

/**
 * @description:
 * @author: sunzhaoning
 * @create: 2019-06-21 10:18
 * @version: V1.0
 **/
@Slf4j
@Service
public class B2bInvoicePackageServiceImpl extends ServiceImpl<B2bInvoicePackageMapper,B2bInvoicePackage> implements B2bInvoicePackageService {

    @Autowired
    private HttpUtil httpUtil;
    @Autowired
    UserService userService;
    @Override
    public IPage getB2bInvoicePackagePage(Page page, B2bInvoicePackage b2bInvoicePackage) {
        return baseMapper.getB2bInvoicePackagePage(page,b2bInvoicePackage);
    }

    @Override
    public RespR getInvoicePackage(B2bInvoicePackage b2bInvoicePackage) {
        List<B2bInvoicePackage> result = new ArrayList<>();

        SimpleDateFormat sdf = new SimpleDateFormat("yyyy-MM-dd");
        //转换时间为 月份
        String month = sdf.format(new Date(Long.parseLong(b2bInvoicePackage.getMakeMonth())));

        Map<String, Object> map = new HashMap<>(3);
        map.put("companyNum", b2bInvoicePackage.getCompanyNum());
        map.put("makeMonth", month);

        String sendResult = httpUtil.getCheckResult(map, CommonConstants.B2B_INVOICE_PACKAGE_V1_API);
        log.info("查询发票打包接口返回：{},请求参数：{}",sendResult, JSON.toJSONString(map));
        //转换结果为jsonobject
        JSONObject sendResultJson = JSON.parseObject(sendResult);
        //获取返回data值
        String dataCode = sendResultJson.getString("data");
        //判断返回值是否为1
        if ("1".equals(dataCode)) {
            String msg = sendResultJson.getString("msg");
            if("[]".equals(msg)){
                return new RespR<>(Boolean.FALSE,"查询结果为空");
            }
            JSONArray array = JSONArray.parseArray(msg);
            if (array.size() > 0) {
                User user = userService.selectUserById(b2bInvoicePackage.getUserId());
                for (Object object : array) {
                    JSONObject msgJson = (JSONObject) object;
                    String companyName = msgJson.getString("companyName");
                    String makeMonth = msgJson.getString("makeMonth");
                    String csvUrl = msgJson.getString("csvUrl");
                    String pdfUrl = msgJson.getString("pdfUrl");
                    Integer csvStatus = msgJson.getInteger("csvStatus");
                    Integer pdfStatus = msgJson.getInteger("pdfStatus");
                    Integer type = msgJson.getInteger("type");
                    b2bInvoicePackage.setCompanyName(companyName);
                    b2bInvoicePackage.setMakeMonth(makeMonth);
                    b2bInvoicePackage.setCsvUrl(csvUrl);
                    b2bInvoicePackage.setPdfUrl(pdfUrl);
                    b2bInvoicePackage.setCsvStatus(csvStatus);
                    b2bInvoicePackage.setPdfStatus(pdfStatus);
                    b2bInvoicePackage.setType(type);
                    b2bInvoicePackage.setUserName(user.getUserName());
                    b2bInvoicePackage.setUserPhone(user.getPhone());
                    b2bInvoicePackage.setUserCompany(user.getCompany());
                    b2bInvoicePackage.setCreateTime(System.currentTimeMillis());
                    this.save(b2bInvoicePackage);
                    result.add(b2bInvoicePackage);
                }
            }
            return new RespR<>(result);
        }else if("2".equals(dataCode)){
            return new RespR<>(Boolean.FALSE,"查询结果为空");
        }
        return new RespR<>(Boolean.FALSE,"查询失败");
    }
}
