package com.jkcredit.invoice.service.impl;

import com.alibaba.fastjson.JSON;
import com.alibaba.fastjson.JSONArray;
import com.alibaba.fastjson.JSONObject;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.jkcredit.invoice.common.CommonConstants;
import com.jkcredit.invoice.mapper.B2bInvoiceQueryMapper;
import com.jkcredit.invoice.model.entity.B2bInvoiceQuery;
import com.jkcredit.invoice.model.entity.CarEtcCard;
import com.jkcredit.invoice.service.B2bInvoiceQueryService;
import com.jkcredit.invoice.service.CarEtcCardService;
import com.jkcredit.invoice.util.DateUtil;
import com.jkcredit.invoice.util.HttpUtil;
import com.jkcredit.invoice.util.RespR;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.text.SimpleDateFormat;
import java.util.*;

/**
 * @description:
 * @author: sunzhaoning
 * @create: 2019-06-19 16:56
 * @version: V1.0
 **/
@Service
@Slf4j
public class B2bInvoiceQueryServiceImpl extends ServiceImpl<B2bInvoiceQueryMapper, B2bInvoiceQuery> implements B2bInvoiceQueryService {

    @Autowired
    private CarEtcCardService carEtcCardService;
    @Autowired
    private HttpUtil httpUtil;

    @Override
    public RespR queryInvoiceToInterface(B2bInvoiceQuery b2bInvoiceQuery) {
        List<B2bInvoiceQuery> b2bInvoiceQueryList = new ArrayList<>();
        //获取ETC卡信息
        List<CarEtcCard> carEtcCards = carEtcCardService.getCarEtcCardByPlateNum(b2bInvoiceQuery.getPlateNum());
        for (CarEtcCard carEtcCard : carEtcCards) {
            //获取ETC卡号
            String cardId = carEtcCard.getCardId();
            //获取企业编号
            String companyNum = b2bInvoiceQuery.getCompanyNum();
            //获取用户id
            Integer userId = b2bInvoiceQuery.getUserId();
            //获取车牌号
            String plateNum = b2bInvoiceQuery.getPlateNum();
            //设置ETC卡号
            b2bInvoiceQuery.setCardId(cardId);

            SimpleDateFormat sdf = new SimpleDateFormat("yyyy-MM-dd HH:mm:ss");

            //开始时间
            String startTime = sdf.format(new Date(Long.parseLong(b2bInvoiceQuery.getStartInvoiceMakeTime())));
            //结束时间
            String endTime = sdf.format(new Date(Long.parseLong(b2bInvoiceQuery.getEndInvoiceMakeTime())));

            Map<String, Object> map = new HashMap<>(5);
            map.put("companyNum", companyNum);
            map.put("cardId", cardId);
            map.put("startInvoiceMakeTime", DateUtil.dateToTime(startTime));
            map.put("endInvoiceMakeTime", DateUtil.dateToTime(endTime));

            String sendResult = httpUtil.getCheckResult(map, CommonConstants.B2B_INVOICE_QUERY_V1_API);
            log.info("查询已开发票接口返回：{},请求参数：{}",sendResult,JSON.toJSONString(map));
            //转换结果为jsonobject
            JSONObject sendResultJson = JSON.parseObject(sendResult);
            //获取返回data值
            String dataCode = sendResultJson.getString("data");
            //判断返回值是否为1
            if ("1".equals(dataCode)) {
                String msg = sendResultJson.getString("msg");
                if("[]".equals(msg)){
                    return new RespR<>(Boolean.FALSE,"查询结果为空");
                }
                JSONArray msgArray = JSON.parseArray(msg);
                for (int j = 0; j < msgArray.size(); j++) {
                    String sellerName = msgArray.getJSONObject(j).getString("sellerName");
                    String sellerTaxpayerCode = msgArray.getJSONObject(j).getString("sellerTaxpayerCode");
                    String invoiceNum = msgArray.getJSONObject(j).getString("invoiceNum");
                    String invoiceCode = msgArray.getJSONObject(j).getString("invoiceCode");
                    String invoiceType = msgArray.getJSONObject(j).getString("invoiceType");
                    String invoiceUrl = msgArray.getJSONObject(j).getString("invoiceUrl");
                    String invoiceMakeTime = msgArray.getJSONObject(j).getString("invoiceMakeTime");
                    String invoiceHtmlUrl = msgArray.getJSONObject(j).getString("invoiceHtmlUrl");
                    Long amount = msgArray.getJSONObject(j).getLong("amount");
                    Long totalAmount = msgArray.getJSONObject(j).getLong("totalAmount");
                    Long totalTaxAmount = msgArray.getJSONObject(j).getLong("totalTaxAmount");
                    String taxRate = msgArray.getJSONObject(j).getString("taxRate");
                    String enStationName = msgArray.getJSONObject(j).getString("enStationName");
                    String exStationName = msgArray.getJSONObject(j).getString("exStationName");
                    String exTime = msgArray.getJSONObject(j).getString("exTime");
                    Long fee = msgArray.getJSONObject(j).getLong("fee");
                    String tradeId = msgArray.getJSONObject(j).getString("tradeId");
                    Integer vehicleType = msgArray.getJSONObject(j).getInteger("vehicleType");
                    //判断是否存在交易id
                    if(baseMapper.getB2bInvoiceByTradeId(tradeId)!=null){
                        //如果存在全部删除更新为新数据
                        baseMapper.deleteByTradeId(tradeId);
                    }
                    b2bInvoiceQuery = new B2bInvoiceQuery();
                    b2bInvoiceQuery.setUserId(userId);
                    b2bInvoiceQuery.setCompanyNum(companyNum);
                    b2bInvoiceQuery.setCardId(cardId);
                    b2bInvoiceQuery.setPlateNum(plateNum);
                    b2bInvoiceQuery.setSellerName(sellerName);
                    b2bInvoiceQuery.setSellerTaxpayerCode(sellerTaxpayerCode);
                    b2bInvoiceQuery.setInvoiceNum(invoiceNum);
                    b2bInvoiceQuery.setInvoiceCode(invoiceCode);
                    b2bInvoiceQuery.setInvoiceType(invoiceType);
                    b2bInvoiceQuery.setInvoiceUrl(invoiceUrl);
                    b2bInvoiceQuery.setInvoiceMakeTime(invoiceMakeTime);
                    b2bInvoiceQuery.setInvoiceHtmlUrl(invoiceHtmlUrl);
                    b2bInvoiceQuery.setAmount(amount);
                    b2bInvoiceQuery.setTotalAmount(totalAmount);
                    b2bInvoiceQuery.setTotalTaxAmount(totalTaxAmount);
                    b2bInvoiceQuery.setTaxRate(taxRate);
                    b2bInvoiceQuery.setEnStationName(enStationName);
                    b2bInvoiceQuery.setExStationName(exStationName);
                    b2bInvoiceQuery.setExTime(exTime);
                    b2bInvoiceQuery.setFee(fee);
                    b2bInvoiceQuery.setTradeId(tradeId);
                    b2bInvoiceQuery.setVehicleType(vehicleType);
                    b2bInvoiceQuery.setStartInvoiceMakeTime(startTime);
                    b2bInvoiceQuery.setEndInvoiceMakeTime(endTime);
                    b2bInvoiceQueryList.add(b2bInvoiceQuery);

                }
                this.saveBatch(b2bInvoiceQueryList);
                return new RespR<>(b2bInvoiceQueryList);
            }
        }

        return new RespR<>(Boolean.FALSE,"查询失败");
    }
}
