package com.jkcredit.invoice.service.impl;

import com.alibaba.fastjson.JSON;
import com.alibaba.fastjson.JSONArray;
import com.jkcredit.invoice.common.CommonConstants;
import com.jkcredit.invoice.model.entity.BillEstimateResult;
import com.jkcredit.invoice.service.BillEstimateResultService;
import com.jkcredit.invoice.util.DateUtil;
import com.jkcredit.invoice.util.RespR;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.data.redis.core.RedisTemplate;
import org.springframework.stereotype.Service;

import java.util.ArrayList;
import java.util.List;
import java.util.regex.Pattern;
import java.util.stream.Collectors;

/**
 * @description:
 * @author: sunzhaoning
 * @create: 2019-07-18 17:27
 * @version: V1.0
 **/
@Slf4j
@Service
public class BillEstimateResultServiceImpl implements BillEstimateResultService {

    @Autowired
    private RedisTemplate redisTemplate;

    @Override
    public RespR save(List<BillEstimateResult> list) {
        //判断读取文件是否为空
        if (list.size() <= 0) {
            return new RespR<>(Boolean.FALSE, "文件内容为空");
        }


        boolean plateNumberIsNull = list.stream().allMatch(billEstimate -> billEstimate.getPlateNum()==null);
        if (plateNumberIsNull) {
            return new RespR<>(Boolean.FALSE, "车牌号为空,请检查文件");
        }

        boolean exTimeIsNull = list.stream().allMatch(billEstimate -> billEstimate.getExTime()==null);
        if (exTimeIsNull) {
            return new RespR<>(Boolean.FALSE, "交易时间为空,请检查文件");
        }

        boolean totalAmountIsNull = list.stream().allMatch(billEstimate -> billEstimate.getTotalAmount()==null);
        if (totalAmountIsNull) {
            return new RespR<>(Boolean.FALSE, "价税合计为空,请检查文件");
        }
        boolean totalTaxAmountIsNull = list.stream().allMatch(billEstimate -> billEstimate.getTotalTaxAmount()==null);
        if (totalTaxAmountIsNull) {
            return new RespR<>(Boolean.FALSE, "税额为空,请检查文件");
        }
        boolean exTimePattern = list.stream().allMatch(billEstimate -> Pattern.matches("^([1-2][0-9][0-9][0-9]-[0-1]{1}[0-9]-[0-3]{1}[0-9])[T](20|21|22|23|[0-1]\\d):[0-5]\\d:[0-5]\\d$", billEstimate.getExTime()));
        if (!exTimePattern) {
            return new RespR<>(Boolean.FALSE, "交易时间格式错误,请检查文件");
        }

        list.stream().forEach(billEstimateResult -> {
            if (redisTemplate.hasKey(CommonConstants.BILL_ESTIMATE_KEY + billEstimateResult.getPlateNum())) {
                //获取redis数据
                List<BillEstimateResult> billEstimateResults;

                billEstimateResults = JSONArray.parseArray(JSON.toJSONString(redisTemplate.opsForValue().get(CommonConstants.BILL_ESTIMATE_KEY +billEstimateResult.getPlateNum())),BillEstimateResult.class);
                redisTemplate.delete(CommonConstants.BILL_ESTIMATE_KEY + billEstimateResult.getPlateNum());
//                //时间格式转换后存储
                String time = DateUtil.dateToTTime(billEstimateResult.getExTime());
                billEstimateResult.setExTime(time);
                //新增数据
                billEstimateResults.add(billEstimateResult);
                //去重
                billEstimateResults = billEstimateResults.stream().distinct().collect(Collectors.toList());
                //保存到redis
                redisTemplate.opsForValue().set(CommonConstants.BILL_ESTIMATE_KEY + billEstimateResult.getPlateNum() + "", billEstimateResults);
            } else {
                List<BillEstimateResult> billEstimateResults = new ArrayList<>();
                //时间格式转换后存储
                String time = DateUtil.dateToTTime(billEstimateResult.getExTime());
                billEstimateResult.setExTime(time);
                billEstimateResults.add(billEstimateResult);
                //保存到redis
                redisTemplate.opsForValue().set(CommonConstants.BILL_ESTIMATE_KEY + billEstimateResult.getPlateNum(), billEstimateResults);
            }
        });
        log.info(JSON.toJSONString(list));
        return new RespR<>(Boolean.TRUE);
    }
}
