package com.jkcredit.invoice.service.impl;

import cn.afterturn.easypoi.excel.entity.ExportParams;
import com.alibaba.fastjson.JSON;
import com.alibaba.fastjson.JSONArray;
import com.jkcredit.invoice.async.CarFreeBillAsync;
import com.jkcredit.invoice.common.CommonConstants;
import com.jkcredit.invoice.model.entity.*;
import com.jkcredit.invoice.model.vo.CarFreeCarrierBillVo;
import com.jkcredit.invoice.service.*;
import com.jkcredit.invoice.util.*;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.data.redis.core.RedisTemplate;
import org.springframework.stereotype.Service;

import javax.servlet.http.HttpServletResponse;
import java.io.IOException;
import java.math.BigDecimal;
import java.text.ParseException;
import java.text.SimpleDateFormat;
import java.util.*;
import java.util.function.Function;
import java.util.stream.Collectors;

/**
 * @description:
 * @author: sunzhaoning
 * @create: 2019-07-19 13:53
 * @version: V1.0
 **/
@Slf4j
@Service
public class BillEstimateServiceImpl implements BillEstimateService {

    @Autowired
    private RedisTemplate redisTemplate;
    @Autowired
    private UserService userService;
    @Autowired
    private InvoiceService invoiceService;
    @Autowired
    private CarFreeCarrierRegisterService carFreeCarrierRegisterService;
    @Autowired
    private CarFreeBillAsync carFreeBillAsync;
    @Autowired
    private NumInvoiceService numInvoiceService;

    @Autowired
    private BatchService batchService;

    private List<BillRelationEstimate> downList = new ArrayList<>();

    private static SimpleDateFormat sdf = new SimpleDateFormat("yyyy-MM-dd HH:mm:ss");

    @Override
    public synchronized RespR saveAndImport(List<BillEstimate> list, User user){
        /*
//        List<BillRelationEstimate> downList = new ArrayList<>();

        try {
            downList.clear();
            //去重
            list = list.stream().distinct().collect(Collectors.toList());
            //校验列表数据
            RespR respR = checkList(list, user.getId());
            log.info("数据校验返回:{}", respR);
            if (respR.getData().equals(Boolean.FALSE)) {
                return respR;
            }
            //获取用户信息根据用户id查询
            user = userService.getById(user.getId());
            if (user.getThreshold().compareTo(new BigDecimal(-1)) == 0) {
                downList = mergeBill(list);
            } else {
                Map<String, BillRelationEstimate> billRelationEstimateHashMap = getInvoiceData(list, user);
                //实时运单时间区间值
                long realTime = -3000;
                //历史运单区间
                long historyTime = -20000;

                List<CarFreeCarrierBillVo> carFreeCarrierBillVos = new ArrayList<>();
                List<CarFreeCarrierBillVo> carFreeCarrierBillHistoryVos = new ArrayList<>();
                List<CarFreeCarrierBillEnd> carFreeCarrierBillEnds = new ArrayList<>();
                List<NumInvoice> numInvoiceList = new ArrayList<>();

                //得到合适比例的运单进行开票
                billRelationEstimateHashMap.forEach((k, v) -> {
                    //判断运单是实时运单还是历史运单
                    long day = DateUtil.getDistanceDays(v.getStartTime(), new Date());
                    //实时运单
                    if (day <= 0 && day >= realTime) {
                        *//**
                         * 构建实时运单实体
                         *//*
                        CarFreeCarrierBillVo carFreeCarrierBillVo = new CarFreeCarrierBillVo();
                        //运单号
                        carFreeCarrierBillVo.setNum(v.getNum());
                        //车牌号
                        carFreeCarrierBillVo.setPlateNumber(v.getPlateNum());
                        //车牌颜色
                        carFreeCarrierBillVo.setPlateColor(v.getPlateColor());
                        //运单金额
                        carFreeCarrierBillVo.setFee(new BigDecimal(v.getFee()));
                        try {
                            //开始时间
                            carFreeCarrierBillVo.setStartTime(sdf.parse(v.getStartTime()));
                            //结束时间
                            carFreeCarrierBillVo.setEndTime(sdf.parse(v.getEndTime()));
                        } catch (ParseException e) {
                            e.printStackTrace();
                        }
                        //运单目的地址
                        carFreeCarrierBillVo.setDestAddr(v.getDestAddr());
                        //运单开始地址
                        carFreeCarrierBillVo.setSourceAddr(v.getSourceAddr());
                        //税号
                        carFreeCarrierBillVo.setTaxPlayerCode(v.getTaxPlayerCode());
                        //是否自动开票
                        carFreeCarrierBillVo.setAutoFlag(CommonConstants.AUTO_INVOICE);
                        //放入到运单开始list
                        carFreeCarrierBillVos.add(carFreeCarrierBillVo);

                        *//**
                         * 构建运单结束实体
                         *//*
                        CarFreeCarrierBillEnd carFreeCarrierBillEnd = new CarFreeCarrierBillEnd();
                        //运单号
                        carFreeCarrierBillEnd.setNum(v.getNum());
                        try {
                            //运单结束时间
                            carFreeCarrierBillEnd.setBillEndTime(sdf.parse(v.getEndTime()));
                        } catch (ParseException e) {
                            e.printStackTrace();
                        }
                        //运单目的地址
                        carFreeCarrierBillEnd.setRealDestAddr(v.getDestAddr());
                        carFreeCarrierBillEnds.add(carFreeCarrierBillEnd);
                        //历史运单
                    } else if (day <= historyTime) {
                        *//**
                         * 构建历史运单实体
                         *//*
                        //调用历史运单接口
                        CarFreeCarrierBillVo carFreeCarrierBillVo = new CarFreeCarrierBillVo();
                        carFreeCarrierBillVo.setNum(v.getNum());
                        carFreeCarrierBillVo.setPlateNumber(v.getPlateNum());
                        carFreeCarrierBillVo.setPlateColor(v.getPlateColor());
                        carFreeCarrierBillVo.setFee(new BigDecimal(v.getFee()));
                        try {
                            carFreeCarrierBillVo.setStartTime(sdf.parse(v.getStartTime()));
                            carFreeCarrierBillVo.setEndTime(sdf.parse(v.getEndTime()));
                        } catch (ParseException e) {
                            e.printStackTrace();
                        }
                        carFreeCarrierBillVo.setDestAddr(v.getDestAddr());
                        carFreeCarrierBillVo.setSourceAddr(v.getSourceAddr());
                        carFreeCarrierBillVo.setTaxPlayerCode(v.getTaxPlayerCode());
                        carFreeCarrierBillVo.setAutoFlag(CommonConstants.AUTO_INVOICE);
                        carFreeCarrierBillHistoryVos.add(carFreeCarrierBillVo);

                        *//**
                         * 构建运单查询发票实体
                         *//*
                        NumInvoice numInvoice = new NumInvoice();
                        numInvoice.setNum(v.getNum());
                        numInvoiceList.add(numInvoice);
                        //不在开票时间内的运单
                    } else {
                        log.info("运单号：{}，不符合运单时间规则，运单信息是:{}", v.getNum(), JSON.toJSONString(v));
                    }
                });
                log.info("实时开始运单集合：{}", JSON.toJSONString(carFreeCarrierBillVos));
                log.info("历史运单开始运单集合：{}", JSON.toJSONString(carFreeCarrierBillHistoryVos));
                log.info("实时结束运单集合：{}", JSON.toJSONString(carFreeCarrierBillEnds));
                log.info("预估开票运单集合：{}", JSON.toJSONString(numInvoiceList));
                *//**
                 * 实时运单开始
                 *//*
                if (carFreeCarrierBillVos.size() > 0) {
                    //生成批次号
                    String batchNumber = "ZYD" + System.currentTimeMillis();

                    String id = UUID.randomUUID().toString();
                    //创建批次信息
                    boolean result = batchService.saveBatch(id, batchNumber, CommonConstants.WAY_BILL_START, user);
                    if (!result) {
                        return new RespR<>(Boolean.FALSE, "创建批次失败");
                    }
                    //调用实时运单开始接口
//                    carFreeBillAsync.startBill(carFreeCarrierBillVos, user, id, batchNumber);

                }

                *//**
                 * 历史运单
                 *//*
                if (carFreeCarrierBillHistoryVos.size() > 0) {
                    String batchNumber = "ZHYD" + System.currentTimeMillis();
                    String id = UUID.randomUUID().toString();
                    //创建批次信息
                    Boolean result = batchService.saveBatch(id, batchNumber, CommonConstants.WAY_BILL_HISTORY_START, user);
                    if (!result) {
                        return new RespR<>(Boolean.FALSE, "创建批次失败");
                    }
                    //调用历史运单接口
//                    carFreeBillAsync.historyBill(carFreeCarrierBillHistoryVos, user, id, batchNumber);
                }
                *//**
                 * 批量开票
                 *//*
//                if (numInvoiceList.size() > 0) {
////                    Thread.sleep(1000*300);
//                    //批量查询发票信息
//                    numInvoiceService.addNumList(numInvoiceList, user.getId());
//                }
                *//**
                 * 实时运单结束
                 *//*
                if (carFreeCarrierBillEnds.size() > 0) {
//                    Thread.sleep(1000*300);
                    String batchNumber = "ZEYD" + System.currentTimeMillis();
                    String id = UUID.randomUUID().toString();
                    //创建批次信息
                    Boolean result = batchService.saveBatch(id, batchNumber, CommonConstants.WAY_BILL_END, user);
                    if (!result) {
                        return new RespR<>(Boolean.FALSE, "创建批次失败");
                    }
                    //调用实时运单结束接口
//                    carFreeBillAsync.endBill(carFreeCarrierBillEnds, user, id, batchNumber);
                }
                downList.clear();
            }
        } catch (Exception e) {
            e.printStackTrace();
        }
        if (downList.size() > 0) {
            return new RespR<>(Boolean.TRUE, "1");
        } else {
            return new RespR<>(Boolean.TRUE, "2");
        }*/
        return null;

    }

    /**
     * 校验数据格式
     *
     * @param list
     * @return
     */
    private RespR checkList(List<BillEstimate> list, Integer userId) {
        log.info("运单预估结果下载:{}", JSON.toJSONString(list));
        try {
            //判断读取文件是否为空
            if (list.size() <= 0) {
                return new RespR<>(Boolean.FALSE, "文件内容为空");
            }

            Map<String, Integer> numMap = new HashMap<>();
            for (BillEstimate b : list) {
                //map.containsKey()   检测key是否重复
                if (numMap.containsKey(b.getNum())) {
                    return new RespR<>(Boolean.FALSE, "文件中存在重复运单号,请检查文件");
                } else {
                    numMap.put(b.getNum(), 1);
                }
            }
            //判断运单号不为空
            boolean numIsNull = list.stream().allMatch(billEstimate -> billEstimate.getNum() == null);
            if (numIsNull) {
                return new RespR<>(Boolean.FALSE, "运单号为空,请检查文件");
            }

            boolean plateNumberIsNull = list.stream().allMatch(billEstimate -> billEstimate.getPlateNum() == null);
            if (plateNumberIsNull) {
                return new RespR<>(Boolean.FALSE, "车牌号为空,请检查文件");
            }

            boolean startIsNull = list.stream().allMatch(billEstimate -> billEstimate.getStartTime() == null);
            if (startIsNull) {
                return new RespR<>(Boolean.FALSE, "开始时间为空,请检查文件");
            }

            boolean endIsNull = list.stream().allMatch(billEstimate -> billEstimate.getEndTime() == null);
            if (endIsNull) {
                return new RespR<>(Boolean.FALSE, "结束时间为空,请检查文件");
            }
            boolean isInTime = list.stream().allMatch(carFreeCarrierBillVo -> DateUtil.getDistanceDays(sdf.format(carFreeCarrierBillVo.getStartTime()), carFreeCarrierBillVo.getEndTime()) <= 0);
            if (!isInTime) {
                return new RespR<>(Boolean.FALSE, "运单结束时间应在开始时间之后，请检查文件");
            }
            //判断车牌颜色是否为蓝色
            boolean isBlueColor = list.stream().allMatch(carFreeCarrierRegister -> carFreeCarrierRegister.getPlateColor().equals("0"));
            if (isBlueColor) {
                return new RespR<>(Boolean.FALSE, "车牌颜色不支持蓝色车牌备案，请检查文件");
            }

            //判断车牌颜色是否符合规则
            boolean isColor = list.stream().allMatch(carFreeCarrierBillVo -> PlateCheckUtil.isNumeric(carFreeCarrierBillVo.getPlateColor()));
            if (!isColor) {
                return new RespR<>(Boolean.FALSE, "车牌颜色格式有误，请检查文件");
            }

            //判断车牌号是否符合规则
            boolean isPlateNumber = list.stream().allMatch(carFreeCarrierBillVo -> PlateCheckUtil.isPlateNumber(carFreeCarrierBillVo.getPlateNum()));
            if (!isPlateNumber) {
                return new RespR<>(Boolean.FALSE, "车牌号码格式有误，请检查文件");
            }

            //判断运单号是否存在
            boolean isNum = list.stream().allMatch(carFreeCarrierBillVo -> redisTemplate.hasKey(CommonConstants.NUM_KEY + "start" + carFreeCarrierBillVo.getNum()));
            if (isNum) {
                return new RespR<>(Boolean.FALSE, "运单号已存在,请检查运单号");
            }

            //判断传入税号是否存在
            boolean isTaxPlayerCode = list.stream().allMatch(carFreeCarrierBillVo -> invoiceService.getInvoiceByDutyParagraph(carFreeCarrierBillVo.getTaxPlayerCode()) != null);
            if (!isTaxPlayerCode) {
                return new RespR<>(Boolean.FALSE, "税号不存在或税号错误，请检查文件");
            }
            //判断车辆是否备案
            boolean isSuccessPlate = list.stream().allMatch(carFreeCarrierBillVo -> carFreeCarrierRegisterService.getRegisterBySuccessPlateNum(carFreeCarrierBillVo.getPlateNum(), carFreeCarrierBillVo.getPlateColor(), userId) == null);
            if (isSuccessPlate) {
                return new RespR<>(Boolean.FALSE, "存在未备案车辆，请检查上传文件");
            }

        } catch (Exception e) {
            e.printStackTrace();
        }
        log.info("导入文件列表:{}", JSON.toJSONString(list));
        return new RespR<>(Boolean.TRUE);
    }

    /**
     * 合并运单和预估数据
     *
     * @param list
     * @return
     */
    private List<BillRelationEstimate> mergeBill(List<BillEstimate> list) {
        List<BillRelationEstimate> billRelationEstimateList = new ArrayList<>();
        //将传入转换为map
        Map<String, BillEstimate> map = list.stream().collect(Collectors.toMap(bill -> bill.getPlateNum() + "|" + bill.getStartTime() + "|" + bill.getEndTime(), Function.identity(), (entity1, entity2) -> entity1));

        //定义返回导出list
        map.forEach((k, v) -> {
            //分割key
            String[] keys = k.split("\\|");
            List<BillEstimateResult> billEstimateResults;
            //从redis获取key的值
            billEstimateResults = JSONArray.parseArray(JSON.toJSONString(redisTemplate.opsForValue().get(CommonConstants.BILL_ESTIMATE_KEY + keys[0])), BillEstimateResult.class);
            //判断结果不为空
            if (billEstimateResults != null) {
                //循环list
                billEstimateResults.stream().forEach(b -> {
                    //判断交易时间是否在开始时间结束时间之内
                    boolean isInvalidTime = DateUtil.isInvalidTime(sdf.format(v.getStartTime()), sdf.format(v.getEndTime()), b.getExTime());
                    //判断返回是否为true 如果是 放入到导出的list中
                    if (isInvalidTime) {
                        BillRelationEstimate billRelationEstimate = new BillRelationEstimate();
                        billRelationEstimate.setNum(v.getNum());
                        billRelationEstimate.setPlateNum(v.getPlateNum());
                        billRelationEstimate.setStartTime(sdf.format(v.getStartTime()));
                        billRelationEstimate.setSourceAddr(v.getSourceAddr());
                        billRelationEstimate.setDestAddr(v.getDestAddr());
                        billRelationEstimate.setFee(v.getFee());
                        billRelationEstimate.setPlateColor(v.getPlateColor());
                        billRelationEstimate.setTaxPlayerCode(v.getTaxPlayerCode());
                        billRelationEstimate.setEndTime(sdf.format(v.getEndTime()));
                        billRelationEstimate.setExTime(b.getExTime());
                        billRelationEstimate.setTotalAmount(b.getTotalAmount());
                        billRelationEstimate.setTotalTaxAmount(b.getTotalTaxAmount());
                        billRelationEstimateList.add(billRelationEstimate);
                    }
                });
            }
        });
        return billRelationEstimateList;
    }

    /**
     * 运单预估结束，开始整合数据实现自动开票 返回适合开票比例的的Map
     *
     * @param list
     * @param user
     * @return
     */
    private Map<String, BillRelationEstimate> getInvoiceData(List<BillEstimate> list, User user) {
        //合并运单和预估结果
        List<BillRelationEstimate> billRelationEstimateList = mergeBill(list);
        Map<String, BillRelationEstimate> billRelationEstimateHashMap = new HashMap<>(16);
        if (billRelationEstimateList.size() > 1) {
            billRelationEstimateList.stream().forEach(b -> {
                if (billRelationEstimateHashMap.containsKey(b.getNum())) {
                    log.info("获取到的map值:{}", billRelationEstimateHashMap.get(b.getNum()));
                    BillRelationEstimate temp = billRelationEstimateHashMap.get(b.getNum());
                    b.setTotalCount(temp.getTotalCount() + 1);
                    if (temp.getAll() == null) {
                        temp.setAll(new BigDecimal(0));
                    }
                    b.setAll(temp.getAll().add(new BigDecimal(b.getTotalTaxAmount())));
                }
                log.info("整合后数据:{}", JSON.toJSONString(b));
                billRelationEstimateHashMap.put(b.getNum(), b);
            });
        }
        if(billRelationEstimateList.size() ==1){
            billRelationEstimateList.stream().forEach(b -> {
                b.setAll(new BigDecimal(b.getTotalTaxAmount()));
                b.setTotalCount(1);
                billRelationEstimateHashMap.put(b.getNum(), b);
            });
        }

        log.info("未处理数据列表：{}", JSON.toJSONString(billRelationEstimateHashMap));

        log.info("用户信息是:{}", JSON.toJSONString(user));
        try {
            billRelationEstimateHashMap.forEach((k, v) -> {
              /*  //判断预估金额不为空并且大于0
                if (v.getAll() != null && v.getAll().compareTo(new BigDecimal(0)) > 0) {
                    //计算 每个运单总价 预估结果数*用户单价
                    BigDecimal count = v.getAll().multiply(ReadExcelUtil.longToBigDecimal(user.getPrice()));
                    log.info("预估结果数*用户单价:{}", count);

                    //计算 用户总消费金额/抵税金额
                    BigDecimal taxAmount = count.divide(v.getAll(), 2, BigDecimal.ROUND_HALF_UP).multiply(new BigDecimal(100));
                    log.info("用户总消费金额/抵税金额:{}", taxAmount);

                    log.info("运单编号：{}，运单比例:{}", k, taxAmount);
                    //判断抵税金额是否小于，如果小于0 移除
                    if (taxAmount.compareTo(user.getThreshold()) < 0) {
                        billRelationEstimateHashMap.remove(k);
                        log.info("运单编号：{},预估金额为小于用户比例", k);
                    }
                } else {
                    billRelationEstimateHashMap.remove(k);
                    log.info("运单编号：{},预估金额为0元，直接跳过", k);
                }*/

            });
        } catch (Exception e) {
            e.printStackTrace();
        }

        log.info("计算后数据列表：{}", JSON.toJSONString(billRelationEstimateHashMap));
        return billRelationEstimateHashMap;
    }

    @Override
    public void download(HttpServletResponse response) {
        log.info("导出数据:{}", JSON.toJSONString(downList));
        if (downList.size() > 0) {
            try {
                //导出文件
                ExportUtil.exportExcel(downList, BillRelationEstimate.class, "运单预估结果", new ExportParams("运单预估结果", "运单预估结果"), response);
            } catch (IOException e) {
                e.printStackTrace();
            }
        }
    }

}
