package com.jkcredit.invoice.service.impl;

import com.alibaba.fastjson.JSON;
import com.alibaba.fastjson.JSONArray;
import com.alibaba.fastjson.JSONObject;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.jkcredit.invoice.common.CommonConstants;
import com.jkcredit.invoice.mapper.CarEtcCardMapper;
import com.jkcredit.invoice.model.entity.CarEtcCard;
import com.jkcredit.invoice.service.CarEtcCardService;
import com.jkcredit.invoice.util.HttpUtil;
import com.jkcredit.invoice.util.RespR;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import java.util.*;

/**
 * @description:
 * @author: sunzhaoning
 * @create: 2019-06-17 14:21
 * @version: V1.0
 **/
@Service
@Slf4j
public class CarEtcCardServiceImpl extends ServiceImpl<CarEtcCardMapper,CarEtcCard> implements CarEtcCardService {

    @Autowired
    private HttpUtil httpUtil;

    @Transactional(rollbackFor = Exception.class)
    @Override
    public RespR etcCardBing(CarEtcCard carEtcCard, String code) {
        Map<String, Object> sendMap = new HashMap<>(3);
        sendMap.put("companyNum", carEtcCard.getCompanyNum());
        sendMap.put("mobile", carEtcCard.getMobile());
        sendMap.put("validCode", code);

        //调用验证码接口
        String sendResult = httpUtil.getCheckResult(sendMap, CommonConstants.CARD_VALID_CODE_V1_API);
        log.info("发送验证码接口返回：{},请求参数：{}",sendResult,JSON.toJSONString(sendMap));
        //转换结果为jsonobject
        JSONObject sendResultJson = JSON.parseObject(sendResult);
        //获取返回data值
        String dataCode = sendResultJson.getString("data");
        //判断返回值是否为1 如果是1 设置状态为成功
        if (!"1".equals(dataCode)) {
            return new RespR<>(Boolean.FALSE,"请检查手机号和验证码或重新开始绑卡");
        }
        //获取返回的msg
        String msg = sendResultJson.getString("msg");
        JSONArray msgJson = JSON.parseArray(msg);
        //省份
        carEtcCard.setProvince(msgJson.getJSONObject(0).getInteger("province"));
        //绑定时间
        carEtcCard.setBindTime(msgJson.getJSONObject(0).getString("bindTime"));
        //卡号
        carEtcCard.setCardId(msgJson.getJSONObject(0).getString("cardId"));
        //卡号id
        carEtcCard.setEtcCardId(msgJson.getJSONObject(0).getString("etcCardId"));
        //卡类型
        carEtcCard.setCardType(msgJson.getJSONObject(0).getInteger("cardType"));
        //车辆类型
        carEtcCard.setVehicleType(msgJson.getJSONObject(0).getInteger("vehicleType"));
        //车牌号
        carEtcCard.setPlateNum(msgJson.getJSONObject(0).getString("plateNum"));
        //车牌颜色
        carEtcCard.setPlateColor(msgJson.getJSONObject(0).getInteger("plateColor"));
        return new RespR<>(this.save(carEtcCard));
    }

    @Override
    public RespR sendCode(CarEtcCard carEtcCard) {
        Map<String, Object> bindMap = new HashMap<>(5);
        bindMap.put("companyNum", carEtcCard.getCompanyNum());
        bindMap.put("mobile", carEtcCard.getMobile());
        bindMap.put("cardIdList", carEtcCard.getCardList());

        //调用绑卡接口
        String bindResult = httpUtil.getCheckResult(bindMap, CommonConstants.CARD_BINDING_V1_API);
        log.info("用户绑卡接口返回：{},请求参数：{}",bindResult,JSON.toJSONString(bindMap));
        //转换结果为jsonobject
        JSONObject bindResultJson = JSON.parseObject(bindResult);
        //获取返回data值
        String dataCode = bindResultJson.getString("data");
        //判断返回值是否为1 如果是1 设置状态为成功
        if (!"1".equals(dataCode)) {
            String message = "发送失败，请稍后重试";
            String msg = bindResultJson.getString("msg");
            String[] msgArray = msg.split(":");
            if (msgArray.length >= 2) {
                message = msgArray[1].replace("\n","");
            }
            return new RespR<>(Boolean.FALSE,message);
        }

        return new RespR<>(Boolean.TRUE);
    }

    @Override
    public IPage getCarEtcCardPage(Page page, CarEtcCard carEtcCard) {
        return baseMapper.getCarEtcCardPage(page, carEtcCard);
    }

    @Override
    public List<CarEtcCard> getCarEtcCardByPlateNum(String plateNum) {
        return baseMapper.getCarEtcCardByPlateNum(plateNum);
    }

    @Override
    public RespR getCardListByInterface(CarEtcCard carEtcCard) {
        if(baseMapper.getCarEtcCardByPlateNum(carEtcCard.getPlateNum())!=null){
            baseMapper.getCarEtcCardByPlateNum(carEtcCard.getPlateNum());
        }
        Map<String, Object> queryMap = new HashMap<>(3);
        queryMap.put("companyNum", carEtcCard.getCompanyNum());
        queryMap.put("vehicleList", carEtcCard.getVehicleList());

        //调用车辆查卡接口
        String queryResult = httpUtil.getCheckResult(queryMap, CommonConstants.CARD_QUERY_CARD_V1_API);
        log.info("用户查卡接口返回：{},请求参数：{}",queryResult,JSON.toJSONString(queryMap));
        //转换结果为jsonobject
        JSONObject queryResultJson = JSON.parseObject(queryResult);
        //获取返回data值
        String dataCode = queryResultJson.getString("data");
        log.info(queryResult);
        //判断返回值是否为1 如果是1 设置状态为成功
        if (!"1".equals(dataCode)) {
            return new RespR<>(Boolean.FALSE,"未查询到车辆对应的ETC卡号信息");
        }
        String msg = queryResultJson.getString("msg");

        if("[]".equals(msg)){
            return new RespR(Boolean.FALSE,"未查询到车辆对应的ETC卡号信息");
        }
        List<CarEtcCard> list = JSONArray.parseArray(msg,CarEtcCard.class);
        return new RespR(list);
    }


}
