package com.jkcredit.invoice.service.impl;

import com.alibaba.fastjson.JSONObject;
import com.jkcredit.invoice.async.CarFreeBillAsync;
import com.jkcredit.invoice.common.CommonConstants;
import com.jkcredit.invoice.model.entity.CarFreeCarrierBillEnd;
import com.jkcredit.invoice.model.entity.CarFreeCarrierRegister;
import com.jkcredit.invoice.model.entity.Invoice;
import com.jkcredit.invoice.model.entity.User;
import com.jkcredit.invoice.model.vo.CarFreeCarrierBillVo;
import com.jkcredit.invoice.service.*;
import com.jkcredit.invoice.util.DateUtil;
import com.jkcredit.invoice.util.PlateCheckUtil;
import com.jkcredit.invoice.util.RespR;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.lang3.StringUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.data.redis.core.RedisTemplate;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import java.text.SimpleDateFormat;
import java.util.*;

/**
 * @description:
 * @author: sunzhaoning
 * @create: 2019-05-31 10:32
 * @version: V1.0
 **/
@Slf4j
@Service
public class CarFreeCarrierBillServiceImpl implements CarFreeCarrierBillService {

    @Autowired
    private UserService userService;

    @Autowired
    private BatchService batchService;

    @Autowired
    private InvoiceService invoiceService;

    @Autowired
    private CarFreeCarrierBillStartService carFreeCarrierBillStartService;

    @Autowired
    private CarFreeCarrierRegisterService carFreeCarrierRegisterService;

    @Autowired
    private RedisTemplate redisTemplate;

    @Autowired
    private CarFreeBillAsync carFreeBillAsync;

    @Transactional(rollbackFor = Exception.class)
    @Override
    public RespR<Boolean> addCarFreeCarrierBillList(List<CarFreeCarrierBillVo> list, Integer userId) {
        //生成批次号
        String batchNumber = "YD" + System.currentTimeMillis();

        JSONObject checkResult = checkRealTimeBillParam(list, "1");
        if (checkResult.getString("code").equals("1")) {
            return new RespR<>(Boolean.FALSE, checkResult.getString("errorMsg"));
        }

        //获取用户信息根据用户id查询
        User user = userService.getById(userId);
       /* if (user.getMoney() <= 0) {
            return new RespR<>(Boolean.FALSE, "余额不足");
        }*/

        Set<CarFreeCarrierRegister> notRegisterPlateNumSet = new HashSet<>();
        list.stream().forEach(bill -> {
            if (carFreeCarrierRegisterService.getRegisterBySuccessPlateNum(bill.getPlateNumber(),bill.getPlateColor(), userId) == null) {
                CarFreeCarrierRegister register = new CarFreeCarrierRegister();
                register.setPlateNumber(bill.getPlateNumber());
                register.setPlateColor(bill.getPlateColor());
                notRegisterPlateNumSet.add(register);
            }
        });

        Map<String, String> registerFailedMap = carFreeCarrierRegisterService.addCarFreeCarrierRegisterList(new ArrayList<>(notRegisterPlateNumSet), userId);
        String id = UUID.randomUUID().toString();

        //创建批次信息
        boolean result = batchService.saveBatch(id, batchNumber, CommonConstants.WAY_BILL_START, user);
        if (!result) {
            return new RespR<>(Boolean.FALSE, "创建批次失败");
        }

        carFreeBillAsync.startBill(list, user, id, batchNumber, registerFailedMap);
        return new RespR<>(Boolean.TRUE);
    }

    private JSONObject checkRealTimeBillParam(List<CarFreeCarrierBillVo> list, String isHistory) {
        JSONObject result = new JSONObject();

        //判断读取文件是否为空
        if (list.size() <= 0) {
            result.put("code", "1");
            result.put("errorMsg", "文件内容为空");
            return result;
        }

        //判断运单号不为空
        boolean numIsNull = list.stream().allMatch(carFreeCarrierBillVo -> carFreeCarrierBillVo.getNum() == null);
        if (numIsNull) {
            result.put("code", "1");
            result.put("errorMsg", "运单号存在空值,请检查文件");
            return result;
        }

        //判断车牌颜色是否符合规则
        boolean isColor = list.stream().allMatch(carFreeCarrierBillVo -> PlateCheckUtil.isNumeric(carFreeCarrierBillVo.getPlateColor()));
        if (!isColor) {
            result.put("code", "1");
            result.put("errorMsg", "车牌颜色格式有误，请检查文件");
            return result;
        }

        //判断车牌颜色是否为蓝色
        boolean isBlueColor = list.stream().allMatch(carFreeCarrierRegister -> carFreeCarrierRegister.getPlateColor().equals("0"));
        if (isBlueColor) {
            result.put("code", "1");
            result.put("errorMsg", "车牌颜色不支持蓝色车牌备案，请检查文件");
            return result;
        }

        //判断传入税号是否存在
        boolean isTaxPlayerCode = list.stream().allMatch(carFreeCarrierBillVo -> invoiceService.getInvoiceByDutyParagraph(carFreeCarrierBillVo.getTaxPlayerCode()) != null);
        if (!isTaxPlayerCode) {
            result.put("code", "1");
            result.put("errorMsg", "税号不存在或税号错误，请检查文件");
            return result;
        }

        SimpleDateFormat sdf = new SimpleDateFormat("yyyy-MM-dd HH:mm:ss");

        Set<String> errorPlateNumberSet = new HashSet<>();
        StringBuilder startTimeLaterThanEndTime = new StringBuilder();
        StringBuilder overTwelveDays = new StringBuilder();
        StringBuilder notRealTimeBill = new StringBuilder();
        StringBuilder existBill = new StringBuilder();
        Map<String, Integer> repeatedNumMap = new HashMap<>();
        StringBuilder repeatedNum = new StringBuilder();
        StringBuilder errorStartTime = new StringBuilder();
        StringBuilder errorEndTime = new StringBuilder();

        for (int i = 0; i < list.size(); i++) {
            // 判断车牌号是否符合规则
            if (!PlateCheckUtil.isPlateNumber(list.get(i).getPlateNumber())) {
                errorPlateNumberSet.add(list.get(i).getPlateNumber());
            }

            if (list.get(i).getStartTime() == null) {
                // 开始时间为空
                errorStartTime.append(list.get(i).getNum()).append(",");
            } else if (list.get(i).getEndTime() == null) {
                // 结束时间为空
                errorEndTime.append(list.get(i).getNum()).append(",");
            } else if (DateUtil.getDistanceDays(sdf.format(list.get(i).getEndTime()), list.get(i).getStartTime()) >= 12000) {
                // 运单结束时间与开始时间最大间隔为12天
                overTwelveDays.append(list.get(i).getNum()).append(",");
            } else if (DateUtil.getDistanceDays(sdf.format(list.get(i).getStartTime()), list.get(i).getEndTime()) > 0) {
                // 运单结束时间应在开始时间之后
                startTimeLaterThanEndTime.append(list.get(i).getNum()).append(",");
            } else if (!PlateCheckUtil.isNowTime(sdf.format(list.get(i).getStartTime()), 2) && isHistory.equals("1")) {
                // 实时运单，开始时间距当前时间大于2天
                notRealTimeBill.append(list.get(i).getNum()).append(",");
            }

            // 判断运单是否上传过
            if (redisTemplate.hasKey(CommonConstants.NUM_KEY + "start" + list.get(i).getNum())) {
                existBill.append(list.get(i).getNum()).append(",");
            }
            // 判断文件中是否存在重复运单号
            if (repeatedNumMap.containsKey(list.get(i).getNum())) {
                repeatedNum.append(list.get(i).getNum()).append(",");
            } else {
                repeatedNumMap.put(list.get(i).getNum(), 1);
            }
        }

        StringBuilder errorMsg = new StringBuilder();
        if (errorPlateNumberSet.size() > 0) {
            Iterator<String> iterator = errorPlateNumberSet.iterator();
            errorMsg.append("车牌号格式错误:");
            while (iterator.hasNext()) {
                errorMsg.append(iterator.next());
                errorMsg.append(",");
            }
        }
        if (startTimeLaterThanEndTime.length() != 0) {
            errorMsg.append("运单结束时间应在开始时间之后:").append(startTimeLaterThanEndTime);
        }
        if (overTwelveDays.length() != 0) {
            errorMsg.append("运单结束时间与开始时间最大间隔为12天:").append(overTwelveDays);
        }
        if (notRealTimeBill.length() != 0) {
            errorMsg.append("时间格式错误或不在规定时间范围:").append(notRealTimeBill);
        }
        if (existBill.length() != 0) {
            errorMsg.append("运单号已存在:").append(existBill);
        }
        if (repeatedNum.length() != 0) {
            errorMsg.append("文件中存在重复运单号:").append(repeatedNum);
        }
        if (errorStartTime.length() != 0) {
            errorMsg.append("开始时间格式错误:").append(errorStartTime);
        }
        if (errorEndTime.length() != 0) {
            errorMsg.append("结束时间格式错误").append(errorEndTime);
        }

        if (errorMsg.length() != 0) {
            result.put("code", "1");
            result.put("errorMsg", errorMsg.toString());
        } else {
            result.put("code", "2");
            result.put("errorMsg", "");
        }
        return result;
    }

    @Transactional(rollbackFor = Exception.class)
    @Override
    public RespR<Boolean> addCarFreeCarrierBillHistoryList(List<CarFreeCarrierBillVo> list, Integer userId) {
        JSONObject checkResult = checkRealTimeBillParam(list, "2");
        if (checkResult.getString("code").equals("1")) {
            return new RespR<>(Boolean.FALSE, checkResult.getString("errorMsg"));
        }

        //获取用户信息根据用户id查询
        User user = userService.getById(userId);
        /*if (user.getMoney() <= 0) {
            return new RespR<>(Boolean.FALSE, "余额不足");
        }*/

        Set<CarFreeCarrierRegister> notRegisterPlateNumSet = new HashSet<>();
        list.stream().forEach(bill -> {
            if (carFreeCarrierRegisterService.getRegisterBySuccessPlateNum(bill.getPlateNumber(),bill.getPlateColor(), userId) == null) {
                CarFreeCarrierRegister register = new CarFreeCarrierRegister();
                register.setPlateNumber(bill.getPlateNumber());
                register.setPlateColor(bill.getPlateColor());
                notRegisterPlateNumSet.add(register);
            }
        });

        Map<String, String> registerFailedMap = carFreeCarrierRegisterService.addCarFreeCarrierRegisterList(new ArrayList<>(notRegisterPlateNumSet), userId);

        //生成批次号
        String batchNumber = "HYD" + System.currentTimeMillis();

        String id = UUID.randomUUID().toString();

        //创建批次信息
        boolean result = batchService.saveBatch(id, batchNumber, CommonConstants.WAY_BILL_HISTORY_START, user);
        if (!result) {
            return new RespR<>(Boolean.FALSE, "创建批次失败");
        }

        carFreeBillAsync.historyBill(list, user, id, batchNumber, registerFailedMap);
        return new RespR<>(Boolean.TRUE);
    }


    @Transactional(rollbackFor = Exception.class)
    @Override
    public RespR<Boolean> addCarFreeCarrierBillEndList(List<CarFreeCarrierBillEnd> list, Integer userId) {
        //获取用户信息根据用户id查询
        User user = userService.getById(userId);
        /*if (user.getMoney() <= 0) {
            return new RespR<>(Boolean.FALSE, "余额不足");
        }*/

        //生成批次号
        String batchNumber = "EYD" + System.currentTimeMillis();
        //判断读取文件是否为空
        if (list.size() <= 0) {
            return new RespR<>(Boolean.FALSE, "文件内容为空");
        }

        //判断运单号不为空
        boolean numIsNull = list.stream().allMatch(carFreeCarrierBillVo -> carFreeCarrierBillVo.getNum() == null);
        if (numIsNull) {
            return new RespR<>(Boolean.FALSE, "运单号为空,请检查文件");
        }

        //判断是否存在重复运单号
        Map<String, Integer> map = new HashMap<>();
        for (CarFreeCarrierBillEnd c : list) {
            //map.containsKey()   检测key是否重复
            if (map.containsKey(c.getNum())) {
                return new RespR<>(Boolean.FALSE, "文件中存在重复运单号,请检查文件");
            } else {
                map.put(c.getNum(), 1);
            }
        }

        //判断运单号是否存在
        boolean isNum = list.stream().allMatch(carFreeCarrierBillEnd -> redisTemplate.hasKey(CommonConstants.NUM_KEY + "end" + carFreeCarrierBillEnd.getNum()));
        if (isNum) {
            return new RespR<>(Boolean.FALSE, "运单号已存在,请检查文件");
        }

        //判断运单号是否存在
        boolean isHaveStartNum = list.stream().allMatch(carFreeCarrierBillEnd -> carFreeCarrierBillStartService.getBillNumByNum(carFreeCarrierBillEnd.getNum(), CommonConstants.SUCCESS) == null);
        if (isHaveStartNum) {
            return new RespR<>(Boolean.FALSE, "运单未开始无法使用结束运单,请检查文件");
        }

        String id = UUID.randomUUID().toString();

        //创建批次信息
        boolean result = batchService.saveBatch(id, batchNumber, CommonConstants.WAY_BILL_END, user);
        if (!result) {
            return new RespR<>(Boolean.FALSE, "创建批次失败");
        }
        carFreeBillAsync.endBill(list, user, id, batchNumber);
        return new RespR<>(Boolean.TRUE);
    }

}
