package com.jkcredit.invoice.service.impl;

import com.alibaba.fastjson.JSON;
import com.alibaba.fastjson.JSONObject;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.google.common.util.concurrent.ThreadFactoryBuilder;
import com.jkcredit.invoice.common.CommonConstants;
import com.jkcredit.invoice.model.entity.CarFreeCarrierRegister;
import com.jkcredit.invoice.model.entity.User;
import com.jkcredit.invoice.mapper.CarFreeCarrierRegisterMapper;
import com.jkcredit.invoice.service.BatchService;
import com.jkcredit.invoice.service.CarFreeCarrierRegisterService;
import com.jkcredit.invoice.service.UserService;
import com.jkcredit.invoice.util.HttpUtil;
import com.jkcredit.invoice.util.PlateCheckUtil;
import com.jkcredit.invoice.util.RespR;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.lang3.StringUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import java.util.*;
import java.util.concurrent.*;

/**
 * @description:
 * @author: sunzhaoning
 * @create: 2019-05-30 14:57
 * @version: V1.0
 **/
@Slf4j
@Service
public class CarFreeCarrierRegisterServiceImpl extends ServiceImpl<CarFreeCarrierRegisterMapper, CarFreeCarrierRegister> implements CarFreeCarrierRegisterService {

    @Autowired
    private UserService userService;

    @Autowired
    private BatchService batchService;

    @Autowired
    private HttpUtil httpUtil;
    @Autowired
    private CarFreeCarrierRegisterService carFreeCarrierRegisterService;

    @Override
    public IPage getCarFreeCarrierRegisterPage(Page page, CarFreeCarrierRegister carFreeCarrierRegister) {
        return baseMapper.getCarFreeCarrierRegisterPage(page, carFreeCarrierRegister);
    }

    @Override
    public Map<String, String> addCarFreeCarrierRegisterList(List<CarFreeCarrierRegister> list, Integer userId) {

        //生成批次号
        String batchNumber = "BA" + System.currentTimeMillis();
        String id = UUID.randomUUID().toString();

        //获取用户信息根据用户id查询
        User user = userService.getById(userId);

        //创建批次信息
        batchService.saveBatch(id, batchNumber, CommonConstants.VEHICLE_REGISTER, user);

        //线程分割数
        int count = 0;
        //获取list大小
        int listSize = list.size();
        if(listSize<=100){
            count = 20;
        }else if(listSize<= 1000&& listSize>100){
            count = 50;
        }else if(listSize<= 12000&& listSize>1000){
            count = 120;
        }else if(listSize<=20000 && listSize> 12000){
            count = 150;
        }else if(listSize<=30000 && listSize> 20000){
            count = 300;
        }else {
            count = 500;
        }
        //线程数
        int threadSize = (listSize / count) + 1;

        ThreadFactory namedThreadFactory = new ThreadFactoryBuilder().setNameFormat("thread-carRegister-%d").build();

        ExecutorService executor = new ThreadPoolExecutor(threadSize, threadSize, 200L, TimeUnit.MILLISECONDS, new LinkedBlockingQueue<>(), namedThreadFactory);
        CountDownLatch countDownLatch = new CountDownLatch(threadSize);
        List<CarFreeCarrierRegister> newList;
        Map<String, String> registerFailedMap = new HashMap<>();
        for (int i = 0; i < threadSize; i++) {
            if ((i + 1) == threadSize) {
                int startIndex = (i * count);
                int endIndex = list.size();
                newList = list.subList(startIndex, endIndex);
            } else {
                int startIndex = i * count;
                ;
                int endIndex = (i + 1) * count;
                newList = list.subList(startIndex, endIndex);
            }

            List<CarFreeCarrierRegister> finalNewList = newList;
            executor.execute(new Thread(() -> {
                if (null != finalNewList) {
                    finalNewList.stream().forEach(carFreeCarrierRegister -> {
                        Map<String, Object> map = new HashMap<>(4);
                        map.put("companyNum","10004616");
                        map.put("plateNumber", carFreeCarrierRegister.getPlateNumber());
                        map.put("plateColor", carFreeCarrierRegister.getPlateColor());

                        //调用接口
                        String registerResult = httpUtil.getCheckResult(map, CommonConstants.VEHICLE_REGISTER_API);
                        // 测试用 不能提交
//                        String registerResult = "{\"data\":1,\"code\":200,\"requestid\":\"20200805114311436,ff4aff34-26c1-4e84-ace6-b66a231e592d\",\"msg\":\"{\\\"info\\\":\\\"车辆备案成功!\\\",\\\"receiveTime\\\":\\\"2020-08-05T11:43:11\\\"}\"}";
//                        String registerResult = "{\"data\":3,\"code\":200,\"requestid\":\"20200805152206578,a743dedc-8464-4765-ba62-45f55c50f10d\",\"msg\":\"备案车辆无卡:备案失败!\\n:803\"}";
                        log.info("车辆备案接口返回：{},请求参数：{}", registerResult, JSON.toJSONString(map));
                        //转换结果为jsonobject
                        JSONObject registerResultJson = JSON.parseObject(registerResult);
                        //获取返回data值
                        String dataCode = registerResultJson.getString("data");

                        //判断返回值是否为1 如果是1 设置状态为成功
                        if ("1".equals(dataCode)) {
                            carFreeCarrierRegister.setErrorMsg("备案成功");
                            carFreeCarrierRegister.setIsSuccess(CommonConstants.SUCCESS);
                        } else {
                            //设置失败
                            log.info(registerResultJson.getString("msg"));
                            carFreeCarrierRegister.setIsSuccess(CommonConstants.ERROR);
                            if (StringUtils.isNotBlank(registerResultJson.getString("msg"))) {
                                carFreeCarrierRegister.setErrorMsg(registerResultJson.getString("msg").replace("\n", ""));
                            } else {
                                carFreeCarrierRegister.setErrorMsg("备案失败！");
                            }
                            registerFailedMap.put(carFreeCarrierRegister.getPlateNumber(), carFreeCarrierRegister.getPlateColor());
                        }

                        carFreeCarrierRegister.setUserId(userId);
                        CarFreeCarrierRegister registerFromDatabase = baseMapper.getCarRegisterByPlateNum(carFreeCarrierRegister);
                        if (registerFromDatabase == null) {
                            carFreeCarrierRegister.setBatchNumber(batchNumber);
                            carFreeCarrierRegister.setCreateTime(System.currentTimeMillis());
                            carFreeCarrierRegister.setBatchId(id);
                            carFreeCarrierRegister.setUserId(user.getId());
                            carFreeCarrierRegister.setUserCompany(user.getCompany());
                            carFreeCarrierRegister.setUserPhone(user.getPhone());
                            carFreeCarrierRegister.setUserName(user.getUserName());
                            carFreeCarrierRegisterService.insert(carFreeCarrierRegister);
                        } else {
                            registerFromDatabase.setIsSuccess(carFreeCarrierRegister.getIsSuccess());
                            registerFromDatabase.setErrorMsg(carFreeCarrierRegister.getErrorMsg());
                            registerFromDatabase.setUpdateTime(System.currentTimeMillis());
                            carFreeCarrierRegisterService.updateById(registerFromDatabase);
                        }
                    });
                }
                countDownLatch.countDown();
            }));
        }

        try {
            countDownLatch.await();
        } catch (InterruptedException e) {
            e.printStackTrace();
        }

        executor.shutdown();
        return registerFailedMap;
    }

    @Override
    public List<CarFreeCarrierRegister> exportList(Integer id, String plateNumber, String isSuccess) {
        return baseMapper.getListByUser(id, plateNumber, isSuccess);
    }

    @Override
    public Integer insert(CarFreeCarrierRegister carFreeCarrierRegister) {
        return baseMapper.insert(carFreeCarrierRegister);
    }

    @Override
    public List<CarFreeCarrierRegister> getRegisterByPlateNum(CarFreeCarrierRegister carFreeCarrierRegister, Integer userId) {
        carFreeCarrierRegister.setIsSuccess(CommonConstants.SUCCESS);
        return baseMapper.getRegisterListByPlateNum(carFreeCarrierRegister, userId);
    }

    @Override
    public CarFreeCarrierRegister getRegisterBySuccessPlateNum(String plateNumber, String plateColor, Integer userId) {
        CarFreeCarrierRegister carFreeCarrierRegister = new CarFreeCarrierRegister();
        carFreeCarrierRegister.setPlateColor(plateColor);
        carFreeCarrierRegister.setPlateNumber(plateNumber);
        carFreeCarrierRegister.setIsSuccess(CommonConstants.SUCCESS);
        carFreeCarrierRegister.setUserId(userId);
        return baseMapper.getRegisterByPlateNum(carFreeCarrierRegister);
    }

}
