package com.jkcredit.invoice.service.impl;

import com.alibaba.fastjson.JSON;
import com.alibaba.fastjson.JSONArray;
import com.alibaba.fastjson.JSONObject;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.google.common.util.concurrent.ThreadFactoryBuilder;
import com.jkcredit.invoice.common.CommonConstants;
import com.jkcredit.invoice.mapper.CardTradeMapper;
import com.jkcredit.invoice.model.entity.CarEtcCard;
import com.jkcredit.invoice.model.entity.CardTrade;
import com.jkcredit.invoice.model.entity.User;
import com.jkcredit.invoice.producer.B2bChargeProducer;
import com.jkcredit.invoice.service.CarEtcCardService;
import com.jkcredit.invoice.service.CardTradeService;
import com.jkcredit.invoice.service.UserService;
import com.jkcredit.invoice.util.DateUtil;
import com.jkcredit.invoice.util.HttpUtil;
import com.jkcredit.invoice.util.RespR;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.data.redis.core.RedisTemplate;
import org.springframework.stereotype.Service;

import java.text.SimpleDateFormat;
import java.util.*;
import java.util.concurrent.*;

/**
 * @description:
 * @author: sunzhaoning
 * @create: 2019-06-19 10:19
 * @version: V1.0
 **/
@Slf4j
@Service
public class CardTradeServiceImpl extends ServiceImpl<CardTradeMapper, CardTrade> implements CardTradeService {

    @Autowired
    private HttpUtil httpUtil;
    @Autowired
    private CarEtcCardService etcCardService;
    @Autowired
    private UserService userService;
    @Autowired
    private RedisTemplate redisTemplate;
    @Autowired
    private B2bChargeProducer b2bChargeProducer;

    @Override
    public IPage getCardTradePage(Page page, CardTrade cardTrade) {
        return baseMapper.getCardTradePage(page, cardTrade);
    }

    @Override
    public RespR<Boolean> updateStatus(Integer id, Integer status) {
        CardTrade cardTrade = new CardTrade();
        cardTrade.setId(id);
        cardTrade.setTradeStatus(status);
        return new RespR<>(baseMapper.updateStatus(cardTrade));
    }

    @Override
    public RespR queryCardTrade(List<CardTrade> cardTradeList) {
        //获取用户信息
        User chargeUser = userService.getById(cardTradeList.get(0).getUserId());
        //判断余额
      /*  if(chargeUser.getMoney()<=0){
            return new RespR<>(Boolean.FALSE,"余额不足");
        }*/
        //判断是否有企业编号
        if(chargeUser.getCompanyNum() == null ||("").equals(chargeUser.getCompanyNum())){
            return new RespR<>(Boolean.FALSE,"请绑定发票信息后操作");
        }
        List<CardTrade> list = new ArrayList<>();

        SimpleDateFormat sdf = new SimpleDateFormat("yyyy-MM-dd HH:mm:ss");

        //线程分割数
        int count = 20;
        //获取list大小
        int listSize = cardTradeList.size();
        //线程数
        int threadSize = (listSize / count) + 1;

        ThreadFactory namedThreadFactory = new ThreadFactoryBuilder().setNameFormat("thread-cardTrade-%d").build();

        ExecutorService executor = new ThreadPoolExecutor(threadSize,threadSize,200L,TimeUnit.MILLISECONDS,new LinkedBlockingQueue<>(),namedThreadFactory);

        CountDownLatch countDownLatch = new CountDownLatch(threadSize);
        List<CardTrade> newList;
        for (int i = 0; i < threadSize; i++) {
            if ((i + 1) == threadSize) {
                int startIndex = (i * count);
                int endIndex = cardTradeList.size();
                newList = cardTradeList.subList(startIndex, endIndex);
            } else {
                int startIndex = i * count;
                ;
                int endIndex = (i + 1) * count;
                newList = cardTradeList.subList(startIndex, endIndex);
            }

            List<CardTrade> finalNewList = newList;
            executor.execute(new Thread(() -> {
                if (null != finalNewList) {
                    finalNewList.stream().forEach(cardTrade -> {
                        String startTime = sdf.format(new Date(Long.parseLong(cardTrade.getStartExTime())));
                        String endTime = sdf.format(new Date(Long.parseLong(cardTrade.getEndExTime())));
                        User user= userService.selectUserById(cardTrade.getUserId());
                        //根据车牌号查询etc卡号
                        List<CarEtcCard> etcCards = etcCardService.getCarEtcCardByPlateNum(cardTrade.getPlateNum());
                        for (CarEtcCard etcCard : etcCards) {
                            cardTrade.setCardId(etcCard.getCardId());
                            //拼接请求参数
                            Map<String, Object> sendMap = new HashMap<>(3);
                            sendMap.put("companyNum", cardTrade.getCompanyNum());
                            sendMap.put("cardId", etcCard.getCardId());
                            //测试暂时修改为3 线上为1
//                        sendMap.put("tradeStatus", 3);
                            sendMap.put("tradeStatus", 1);
                            sendMap.put("startExTime", DateUtil.dateToTime(startTime));
                            sendMap.put("endExTime", DateUtil.dateToTime(endTime));

                            //调用接口
                            String sendResult = httpUtil.getCheckResult(sendMap, CommonConstants.CARD_TRADE_V1_API);
                            log.info("查询交易接口返回：{},请求参数：{}",sendResult,JSON.toJSONString(sendMap));
                            //转换结果为jsonobject
                            JSONObject sendResultJson = JSON.parseObject(sendResult);
                            //获取返回data值
                            String dataCode = sendResultJson.getString("data");
                            //判断返回值是否为1 如果是1 设置状态为成功
                            if ("1".equals(dataCode)) {
                                //从返回值中获取返回信息
                                String msg = sendResultJson.getString("msg");
                                //将msg转换为json集合
                                JSONArray msgArray = JSON.parseArray(msg);
                                for (int j = 0; j < msgArray.size(); j++) {
                                    //获取卡号
                                    String cardId = msgArray.getJSONObject(j).getString("cardId");
                                    //获取价格
                                    Long fee = msgArray.getJSONObject(j).getLong("fee");
                                    //获取时间
                                    String exTime = msgArray.getJSONObject(j).getString("exTime");
                                    //获取交易id
                                    String tradeId = msgArray.getJSONObject(j).getString("tradeId");
                                    if (!redisTemplate.hasKey(CommonConstants.B2B_CHARGE_KEY + cardId + "_" + DateUtil.getMonth(DateUtil.dateFormate(exTime)))) {
                                        b2bChargeProducer.send(user);
                                        log.warn("计费日志:{},计费用户id:{}",JSON.toJSONString(msgArray.getJSONObject(j)), user.getId());
                                        redisTemplate.opsForValue().set(CommonConstants.B2B_CHARGE_KEY + cardId + "_" + DateUtil.getMonth(DateUtil.dateFormate(exTime)), cardId + "_" + DateUtil.getMonth(DateUtil.dateFormate(exTime)));
                                    }
                                    CardTrade cardTradeObject = new CardTrade();
                                    cardTradeObject.setUserId(cardTrade.getUserId());
                                    cardTradeObject.setPlateNum(cardTrade.getPlateNum());
                                    cardTradeObject.setCompanyNum(cardTrade.getCompanyNum());
                                    cardTradeObject.setCardId(etcCard.getCardId());
                                    cardTradeObject.setCardId(cardId);
                                    cardTradeObject.setFee(fee);
                                    cardTradeObject.setExTime(exTime);
                                    cardTradeObject.setTradeId(tradeId);
                                    cardTradeObject.setStartExTime(startTime);
                                    cardTradeObject.setEndExTime(endTime);
                                    //设置状态为请求成功
                                    cardTradeObject.setIsSuccess(CommonConstants.SUCCESS);
                                    cardTradeObject.setTradeStatus(CommonConstants.IN_TRADE_STATUS);
                                    cardTradeObject.setCreateTime(System.currentTimeMillis());
                                    list.add(cardTradeObject);
                                    //插入数据库
                                    this.save(cardTradeObject);
                                }
                            } else {
                                //设置状态请求失败
                                cardTrade.setIsSuccess(CommonConstants.ERROR);
                                cardTrade.setTradeStatus(CommonConstants.NO_TRADE_STATUS);
                                cardTrade.setCreateTime(System.currentTimeMillis());
                                this.save(cardTrade);
                            }
                        }

                    });
                }
                countDownLatch.countDown();
            }));
        }
        try {
            countDownLatch.await();
        } catch (InterruptedException e) {
            e.printStackTrace();
        }

        executor.shutdown();
        if(list.size()<=0){
            return new RespR<>(Boolean.FALSE,"查询结果为空");
        }
        return new RespR<>(list);
    }

    @Override
    public List<CardTrade> getTradeIdList(CardTrade cardTrade) {
        return baseMapper.getTradeIdList(cardTrade);
    }
}
