package com.jkcredit.invoice.service.impl;

import cn.hutool.core.util.StrUtil;
import com.alibaba.fastjson.JSON;
import com.alibaba.fastjson.JSONArray;
import com.alibaba.fastjson.JSONObject;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.jkcredit.invoice.common.CommonConstants;
import com.jkcredit.invoice.mapper.InvoiceMapper;
import com.jkcredit.invoice.model.entity.Invoice;
import com.jkcredit.invoice.model.entity.User;
import com.jkcredit.invoice.service.InvoiceService;
import com.jkcredit.invoice.service.UserService;
import com.jkcredit.invoice.util.HttpUtil;
import com.jkcredit.invoice.util.RespR;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.lang3.StringUtils;
import org.apache.ibatis.annotations.Param;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.util.HashMap;
import java.util.List;
import java.util.Map;

/**
 * @description:
 * @author: sunzhaoning
 * @create: 2019-05-30 09:16
 * @version: V1.0
 **/
@Service
@Slf4j
public class InvoiceServiceImpl extends ServiceImpl<InvoiceMapper, Invoice> implements InvoiceService {

    @Autowired
    private UserService userService;

    @Autowired
    private HttpUtil httpUtil;

    @Override
    public IPage getInvoicesPage(Page page, Invoice invoice) {
        return baseMapper.getInvoicesPage(page, invoice);
    }

    @Override
    public Invoice getInvoiceByDutyParagraph(@Param(value = "dutyParagraph") String dutyParagraph) {
        return baseMapper.getInvoiceByDutyParagraph(dutyParagraph);
    }

    @Override
    public List<Invoice> getByUserId(Integer userId) {
        return baseMapper.getByUserId(userId);
    }

    @Override
    public RespR saveInvoice(Invoice invoice) {
        Integer userId = invoice.getUserId();
        //根据用户ID查询用户信息
        User user = userService.getById(userId);
        String companyNum = companyQuery(invoice.getDutyParagraph());
        if (StringUtils.isBlank(companyNum)) {
            //判断用户角色是不是自有车用户 如果是自有车查询发票张数
            if (StrUtil.equals(user.getRoleId(), "3")) {
                List<Invoice> invoiceList = baseMapper.getByUserId(userId);
                //如果发票张数大于等于1张返回已存在
                if (invoiceList.size() >= 1) {
                    return new RespR<>(Boolean.FALSE, "自有车用户仅能添加一张发票信息");
                }
                Map<String, Object> map = new HashMap<>(32);
                map.put("name", user.getCompany());
                //map.put("taxplayerCode", user.getDutyParagraph());
                map.put("companyType", "1");
                map.put("operatingRangeType", invoice.getOperatingRangeType());
                map.put("contact", invoice.getContact());
                map.put("tel", invoice.getContactTel());
                map.put("emergencyContact", invoice.getContact());
                map.put("emergencyTel", invoice.getContactTel());
                map.put("buyerName", invoice.getCompany());
                map.put("buyerTaxpayerCode", invoice.getDutyParagraph());
                map.put("buyerAddr", invoice.getAddress());
                map.put("buyerTel", invoice.getBuyerTel());
                map.put("buyerBank", invoice.getBankAddress());
                map.put("buyerBankAccount", invoice.getBankNumber());

                //调用接口
                String result = httpUtil.getCheckResult(map, CommonConstants.COMPANY_ADD_V1_API);
                //转换结果为jsonobject
                JSONObject resultJson = JSON.parseObject(result);
                log.info("新增企业接口返回：{},请求参数：{}", result, map);
                //获取返回data值
                String dataCode = resultJson.getString("data");
                //判断返回值是否为1 如果是1 设置状态为成功
                if ("1".equals(dataCode)) {
                    String msg = resultJson.getString("msg");
                    JSONObject msgJson = JSON.parseObject(msg);
                    //如果成功更新企业编号
                    user.setCompanyNum(msgJson.getString("companyNum"));
                    userService.updateCompanyNumById(user);
                } else {
                    //设置失败
                    return new RespR<>(Boolean.FALSE, "发票添加失败");
                }

            } else if (StrUtil.equals(user.getRoleId(), "2") || StrUtil.equals(user.getRoleId(), "4")) {
                Invoice invoiceInDatabase = getInvoiceByDutyParagraph(invoice.getDutyParagraph());
                if (invoiceInDatabase != null) {
                    return new RespR<>(Boolean.FALSE, "已存在税号相同的发票信息");
                }

                Map<String, Object> map = new HashMap<>(32);
                map.put("name", invoice.getCompany());
                map.put("taxplayerCode", invoice.getDutyParagraph());
                if (StrUtil.equals(user.getRoleId(), "2")) {
                    map.put("companyType", "3");
                } else {
                    map.put("companyType", "4");
                }

                map.put("operatingRangeType",invoice.getOperatingRangeType());
                map.put("contact", invoice.getContact());
                map.put("tel",invoice.getContactTel());
                map.put("emergencyContact", invoice.getContact());
                map.put("emergencyTel", invoice.getContactTel());
                map.put("buyerName", invoice.getCompany());
                map.put("buyerTaxpayerCode", invoice.getDutyParagraph());
                map.put("buyerAddr", invoice.getAddress());
                map.put("buyerTel", invoice.getBuyerTel());
                map.put("buyerBank", invoice.getBankAddress());
                map.put("buyerBankAccount", invoice.getBankNumber());

                //调用接口
                String result = httpUtil.getCheckResult(map, CommonConstants.COMPANY_ADD_V1_API);
//                String result = "{\"code\":\"200\",\"data\":\"1\",\"requestId\":\"20201225175012733,8bebef9b-d273-4a15-acee-0c8780f5d8ef\",\"msg\":{\"info\":\"成功\",\"receiveTime\":\"2020-12-25T17:50:12.953\",\"companyNum\":\"11119197\",\"insertTime\":\"2020-12-25T17:50:12.870\"}}";
                //转换结果为jsonobject
                JSONObject resultJson = JSON.parseObject(result);
                log.info("新增企业接口返回：{},请求参数：{}", result, map);
                //获取返回data值
                String dataCode = resultJson.getString("data");
                //判断返回值是否为1 如果是1 设置状态为成功
                if (!"1".equals(dataCode)) {
                    //设置失败
                    return new RespR<>(Boolean.FALSE, "发票添加失败");
                }
            }
        }

        //插入时间
        invoice.setCreateTime(System.currentTimeMillis());
        if (user != null) {
            //插入用户信息
            invoice.setUserCompany(user.getCompany());
            invoice.setUserName(user.getUserName());
            invoice.setUserPhone(user.getPhone());
        }
        return new RespR<>(this.save(invoice));
    }

    private String companyQuery(String taxPlayerCode) {
        Map<String, Object> companyQueryMap = new HashMap<>();
        companyQueryMap.put("taxplayerCode", taxPlayerCode);
        companyQueryMap.put("companyName", "");
        String companyQueryResult = httpUtil.getCheckResult(companyQueryMap, CommonConstants.COMPANY_QUERY_V1_API);
        JSONObject companyQueryJson = JSON.parseObject(companyQueryResult);
        log.info("企业查询接口返回：{},请求参数：{}", companyQueryResult, companyQueryMap);
        String dataCode = companyQueryJson.getString("data");
        if ("1".equals(dataCode)) {
            String msg = companyQueryJson.getString("msg");
            JSONArray companyArray = JSONArray.parseArray(msg);
            if (companyArray.size() == 1) {
                JSONObject company = JSON.parseObject(companyArray.getString(0));
                return company.getString("companyNum");
            }
        }
        return "";
    }
}
