package com.jkcredit.invoice.service.impl;

import com.alibaba.fastjson.JSON;
import com.alibaba.fastjson.JSONArray;
import com.alibaba.fastjson.JSONObject;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.google.common.util.concurrent.ThreadFactoryBuilder;
import com.jkcredit.invoice.common.CommonConstants;
import com.jkcredit.invoice.mapper.NumInvoiceMapper;
import com.jkcredit.invoice.model.entity.*;
import com.jkcredit.invoice.producer.ChargeProducer;
import com.jkcredit.invoice.service.*;
import com.jkcredit.invoice.util.HttpUtil;
import com.jkcredit.invoice.util.RespR;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.data.redis.core.RedisTemplate;
import org.springframework.stereotype.Service;
import org.springframework.transaction.PlatformTransactionManager;
import org.springframework.transaction.TransactionDefinition;
import org.springframework.transaction.TransactionStatus;
import org.springframework.transaction.support.DefaultTransactionDefinition;

import java.util.*;
import java.util.concurrent.*;
import java.util.stream.Collectors;

/**
 * @description:
 * @author: sunzhaoning
 * @create: 2019-06-03 17:09
 * @version: V1.0
 **/
@Slf4j
@Service
public class NumInvoiceServiceImpl extends ServiceImpl<NumInvoiceMapper, NumInvoice> implements NumInvoiceService {

    @Autowired
    private UserService userService;
    @Autowired
    private BatchService batchService;
    @Autowired
    private HttpUtil httpUtil;
    @Autowired
    private CarFreeCarrierBillStartService carFreeCarrierBillStartService;
    @Autowired
    private SearchInvoiceResultService searchInvoiceResultService;
    @Autowired
    private RedisTemplate redisTemplate;
    @Autowired
    private RealChargeService realChargeService;

    @Autowired
    private PlatformTransactionManager transactionManager;

    @Override
    public IPage getNumInvoicePage(Page page, NumInvoice numInvoice) {
        return baseMapper.getNumInvoicePage(page, numInvoice);
    }

    @Override
    public RespR<Boolean> addNumList(List<CarFreeCarrierBillStart> list, Integer userId) {
        //获取用户信息根据用户id查询
        User user = userService.getById(userId);
        //生成批次号
        String batchNumber = "NI" + System.currentTimeMillis();
        //判断读取文件是否为空
        if (list.size() <= 0) {
            return new RespR<>(Boolean.FALSE, "文件内容为空");
        }

        String id = UUID.randomUUID().toString();

        //创建批次信息
        boolean result = batchService.saveBatch(id, batchNumber, CommonConstants.WAY_BILL_NUM_FIND_INVOICE, user);
        if (!result) {
            return new RespR<>(Boolean.FALSE, "创建批次失败");
        }

        list.stream().forEach(billStart -> {
            DefaultTransactionDefinition def = new DefaultTransactionDefinition();
            def.setPropagationBehavior(TransactionDefinition.PROPAGATION_REQUIRED);
            TransactionStatus status = transactionManager.getTransaction(def);
            try {
                Map<String, Object> map = new HashMap<>(4);
                map.put("companyNum", "10004616");
                map.put("num", billStart.getNum());

                NumInvoice numInvoice = new NumInvoice();
                numInvoice.setNum(billStart.getNum());

                //调用接口
                String registerResult = httpUtil.getCheckResult(map, CommonConstants.WAY_BILL_NUM_FIND_INVOICE_API);
//                        String registerResult = getTestResult();
                log.info("运单查询发票接口返回：{}，请求参数：{}", registerResult, JSON.toJSONString(map));
                //转换结果为jsonobject
                JSONObject registerResultJson = JSON.parseObject(registerResult);
                //获取返回data值
                String dataCode = registerResultJson.getString("data");
                //判断返回值是否为1 如果是1 设置状态为成功
                if ("1".equals(dataCode)) {

                    String returnStr = registerResultJson.getString("msg");

                    JSONObject returnStrJson = JSON.parseObject(returnStr);
                    //获取接口返回结果
                    String numResult = returnStrJson.getString("result");
                    //获取接口状态
                    String waybillStatus = returnStrJson.getString("waybillStatus");

                    if ("[]".equals(numResult)) {
                        //如果状态是开票中，更新运单状态
                        if (("2").equals(waybillStatus)) {
                            //更新运单状态
                            carFreeCarrierBillStartService.updateStatus(billStart.getNum(), CommonConstants.INVOICE_PROCESS);
                        }
                    }

                    //如果状态是开票中，更新运单状态
                    if (("2").equals(waybillStatus)) {
                        //更新运单状态
                        carFreeCarrierBillStartService.updateStatus(billStart.getNum(), CommonConstants.INVOICE_PROCESS);
                    } else if (("3").equals(waybillStatus)) {
                        //更新运单状态
                        carFreeCarrierBillStartService.updateStatus(billStart.getNum(), CommonConstants.INVOICE_FINISHED);
                    }
                    numInvoice.setIsSuccess(CommonConstants.SUCCESS);
                    //转换为结果实体对象 并插入数据库 实现计费功能
                    List<SearchInvoiceResult> ts = JSONArray.parseArray(numResult, SearchInvoiceResult.class);
                    List<SearchInvoiceResult> newSearchInvoiceResultList;
                    if (ts.size() > 0) {
                        //去重列表
                        newSearchInvoiceResultList = ts.stream().collect(Collectors.collectingAndThen(Collectors.toCollection(
                                () -> new TreeSet<>(Comparator.comparing(SearchInvoiceResult::getTransactionId))),
                                ArrayList::new));
                        User userForBalance = userService.getById(userId);
                        //Long balance = userForBalance.getMoney();
                        //计费
                        for (SearchInvoiceResult t : newSearchInvoiceResultList) {
                            List<SearchInvoiceResult> searchInvoiceResultList = searchInvoiceResultService.getNumInvoiceByTransactionId(t.getTransactionId());
                            if (searchInvoiceResultList.size() == 0) {
                               // balance = balance - userForBalance.getPrice();
                                // 实际扣费记录
                                RealCharge realCharge = new RealCharge();
                                realCharge.setCreateTime(System.currentTimeMillis());
                                realCharge.setNum(t.getWaybillNum());
                                realCharge.setTransactionId(t.getTransactionId());
                                //realCharge.setPrice(userForBalance.getPrice());
                                realCharge.setUserId(userForBalance.getId());
                                realChargeService.save(realCharge);

                                log.warn("计费日志:{},计费用户id:{}", JSON.toJSONString(realCharge), userId);
                            }
                        }
                       // userForBalance.setMoney(balance);
                        userService.updateById(userForBalance);
                        ts.stream().forEach(t -> {
                            List<SearchInvoiceResult> searchInvoiceResultList = searchInvoiceResultService.getInvoiceByInvoiceNum(t.getInvoiceNum(), t.getInvoiceCode());
                            if (searchInvoiceResultList.size() == 0) {
                                t.setUserId(userId);
                                t.setCreateTime(System.currentTimeMillis());
                                //插入发票信息
                                searchInvoiceResultService.save(t);
                                log.info("插入的发票信息:{}", JSON.toJSONString(t));
                            }
                        });
                    }

                } else {
                    //设置失败
                    numInvoice.setIsSuccess(CommonConstants.ERROR);
                }

                numInvoice.setBatchNumber(batchNumber);
                numInvoice.setCreateTime(System.currentTimeMillis());
                numInvoice.setBatchId(id);
                numInvoice.setUserId(user.getId());
                numInvoice.setUserCompany(user.getCompany());
                numInvoice.setUserPhone(user.getPhone());
                numInvoice.setUserName(user.getUserName());
                baseMapper.insert(numInvoice);

                transactionManager.commit(status);
            } catch (Exception e) {
                log.error("发票查询异常，异常信息：{}", e.getMessage());
                transactionManager.rollback(status);
            }
        });

        return new RespR<>(Boolean.TRUE);
    }

    // todo 测试后删除
    private String getTestResult() {
        Random random = new Random();
        int ends = random.nextInt(99);
        String lastNumber = String.format("%02d", ends);
        String transactionId = System.currentTimeMillis() + lastNumber;
        String result = "{\"data\":1,\"code\":200,\"requestid\":\"20190429103117960,b8f80beb-efa8-494e-827f-02f1c1918ebb\",\"msg\":\"{\\\"info\\\":\\\"成功.\\\",\\\"receiveTime\\\":\\\"2019-04-29T10:30:34\\\",\\\"result\\\":[{\\\"invoiceNum\\\":\\\"" + transactionId + "\\\",\\\"invoiceCode\\\":\\\"" + transactionId + "\\\",\\\"invoiceMakeTime\\\":\\\"2019-04-16T17:24:10\\\",\\\"invoiceUrl\\\":\\\"https://kp.txffp.com/fp/d?d=599963096511912471553E0EECB05E79687E6449B484E7165A99A8A21A714813\\\",\\\"invoiceHtmlUrl\\\":\\\"https://kp.txffp.com/openInvoice/invoice/previewInvoicetxf.html?param=ZnBkbT0wMTQwMDE3MDAxMTImZnBobT0wMjk2OTE1OQ==\\\",\\\"enStation\\\":\\\"大同站\\\",\\\"exStation\\\":\\\"朔州东站\\\",\\\"exTime\\\":\\\"2018-06-30T06:56:06\\\",\\\"fee\\\":10500,\\\"totalTaxAmount\\\":303,\\\"plateNum\\\":\\\"晋B80309\\\",\\\"vehicleType\\\":13,\\\"waybillNum\\\":\\\"20190416\\\",\\\"waybillStatus\\\":2,\\\"waybillStartTime\\\":\\\"2019-04-15T05:00:00\\\",\\\"waybillEndTime\\\":\\\"2019-04-16T08:00:00\\\",\\\"totalAmount\\\":10386,\\\"taxRate\\\":0.03,\\\"invoiceType\\\":\\\"消费发票\\\",\\\"amount\\\":10083,\\\"sellerName\\\":\\\"山西交通控股集团有限公司\\\",\\\"sellerTaxpayerCode\\\":\\\"91140000MA0JTP7A7K\\\",\\\"transactionId\\\":\\\"" + transactionId + "\\\"}],\\\"plateNum\\\":\\\"京A11111\\\",\\\"vehicleType\\\":null,\\\"waybillNum\\\":\\\"20190416\\\",\\\"waybillStatus\\\":3,\\\"waybillStartTime\\\":\\\"2019-04-15T05:00:00\\\",\\\"waybillEndTime\\\":\\\"2019-04-16T08:00:00\\\"}\"}";
        return result;
    }
}
