package com.jkcredit.invoice.service.impl;

import cn.afterturn.easypoi.excel.ExcelExportUtil;
import cn.afterturn.easypoi.excel.entity.TemplateExportParams;
import com.alibaba.fastjson.JSON;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.jkcredit.invoice.common.CommonConstants;
import com.jkcredit.invoice.mapper.SearchInvoiceResultMapper;
import com.jkcredit.invoice.model.entity.SearchInvoiceResult;
import com.jkcredit.invoice.model.entity.User;
import com.jkcredit.invoice.model.vo.SearchInvoiceResultVo;
import com.jkcredit.invoice.service.SearchInvoiceResultService;
import com.jkcredit.invoice.service.UserService;
import com.jkcredit.invoice.util.ReadExcelUtil;
import lombok.extern.slf4j.Slf4j;
import org.apache.poi.ss.usermodel.Workbook;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import javax.servlet.http.HttpServletResponse;
import java.io.*;
import java.math.BigDecimal;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
@Slf4j
@Service
public class SearchInvoiceResultServiceImpl extends ServiceImpl<SearchInvoiceResultMapper, SearchInvoiceResult> implements SearchInvoiceResultService {

    @Autowired
    private UserService userService;

    @Override
    public IPage getSearchInvoiceResultPage(Page page, SearchInvoiceResult searchInvoiceResult, String batchNumber) {
        IPage<SearchInvoiceResult> pages = baseMapper.getSearchInvoiceResultPage(page, searchInvoiceResult, batchNumber);
        log.info("查询结果：{}",JSON.toJSON(pages));
        return baseMapper.getSearchInvoiceResultPage(page, searchInvoiceResult, batchNumber);
    }

    @Override
    public IPage getDailyBilling(Page page, Integer id) {
        return baseMapper.getDailyBilling(page, id);
    }

    @Override
    public void deleteByTransactionId(String transactionId) {
        baseMapper.deleteByTransactionId(transactionId);
    }

    @Override
    public List<SearchInvoiceResultVo> getInvoiceList(Integer userId, String num, String batchNumber, Long startTime, Long endTime) {
        List<SearchInvoiceResultVo> searchInvoiceResultVos = new ArrayList<>();
        //查询数据库返回结果
        List<SearchInvoiceResult> list = baseMapper.getInvoiceList(userId, num, batchNumber, startTime, endTime, null);
        list.stream().forEach(searchInvoiceResult -> {
            SearchInvoiceResultVo searchInvoiceResultVo = new SearchInvoiceResultVo();
            searchInvoiceResultVo.setInvoiceNum(searchInvoiceResult.getInvoiceNum());
            searchInvoiceResultVo.setInvoiceCode(searchInvoiceResult.getInvoiceCode());
            searchInvoiceResultVo.setInvoiceMakeTime(searchInvoiceResult.getInvoiceMakeTime());
            searchInvoiceResultVo.setInvoiceUrl(searchInvoiceResult.getInvoiceUrl());
            searchInvoiceResultVo.setEnStation(searchInvoiceResult.getEnStation());
            searchInvoiceResultVo.setExStation(searchInvoiceResult.getExStation());
            searchInvoiceResultVo.setExTime(searchInvoiceResult.getExTime());
            searchInvoiceResultVo.setFee(ReadExcelUtil.longToBigDecimal(searchInvoiceResult.getFee()));
            searchInvoiceResultVo.setTotalTaxAmount(ReadExcelUtil.longToBigDecimal(searchInvoiceResult.getTotalTaxAmount()));
            searchInvoiceResultVo.setPlateNum(searchInvoiceResult.getPlateNum());
            searchInvoiceResultVo.setVehicleType(searchInvoiceResult.getVehicleType());
            searchInvoiceResultVo.setWaybillNum(searchInvoiceResult.getWaybillNum());
            searchInvoiceResultVo.setWaybillStartTime(searchInvoiceResult.getWaybillStartTime());
            searchInvoiceResultVo.setWaybillEndTime(searchInvoiceResult.getWaybillEndTime());
            searchInvoiceResultVo.setTotalAmount(ReadExcelUtil.longToBigDecimal(searchInvoiceResult.getTotalAmount()));
            searchInvoiceResultVo.setTaxRate(new BigDecimal(searchInvoiceResult.getTaxRate()).setScale(2, BigDecimal.ROUND_HALF_UP).doubleValue());
            searchInvoiceResultVo.setAmount(ReadExcelUtil.longToBigDecimal(searchInvoiceResult.getAmount()));
            searchInvoiceResultVo.setSellerName(searchInvoiceResult.getSellerName());
            searchInvoiceResultVo.setSellerTaxpayerCode(searchInvoiceResult.getSellerTaxpayerCode());
            searchInvoiceResultVo.setTransactionId(searchInvoiceResult.getTransactionId());
            searchInvoiceResultVos.add(searchInvoiceResultVo);
        });
        return searchInvoiceResultVos;
    }

    @Override
    public Workbook exportExcelToTemplate(Integer userId, String num, String batchNumber, Long startTime, Long endTime) {
        //获取用户信息
        User user = userService.selectUserById(userId);
        //调用统计接口
        SearchInvoiceResult searchInvoiceResult = new SearchInvoiceResult();
        searchInvoiceResult.setUserId(userId);
        searchInvoiceResult.setWaybillNum(num);
        searchInvoiceResult.setStartTime(startTime);
        searchInvoiceResult.setEndTime(endTime);
        //返回统计信息
        searchInvoiceResult = statisticsData(searchInvoiceResult, batchNumber);
        //调用查询列表接口
        List<SearchInvoiceResultVo> list = getInvoiceList(userId, num, batchNumber, startTime, endTime);

        TemplateExportParams params = new TemplateExportParams(
                CommonConstants.BASE_PATH + CommonConstants.TEMPLATE_PATH+"发票列表模板.xlsx");
        BigDecimal chargeCount = new BigDecimal(Long.parseLong(searchInvoiceResult.getTransactionId()));
        //BigDecimal price = ReadExcelUtil.longToBigDecimal(user.getPrice());
        //统计数据
        Map<String, Object> map = new HashMap<>(8);
        map.put("amount", ReadExcelUtil.longToBigDecimal(searchInvoiceResult.getFee()));
        map.put("taxAmount", ReadExcelUtil.longToBigDecimal(searchInvoiceResult.getTotalTaxAmount()));
        map.put("count", searchInvoiceResult.getTransactionId());
        //map.put("money", chargeCount.multiply(price).setScale(2, BigDecimal.ROUND_HALF_UP).doubleValue());
        //列表数据
        List<Map<String, Object>> listMap = new ArrayList<>();
        for (int i = 0; i < list.size(); i++) {
            Map<String, Object> lm = new HashMap<>(32);
            lm.put("invoiceNum", list.get(i).getInvoiceNum());
            lm.put("invoiceCode", list.get(i).getInvoiceCode());
            lm.put("invoiceMakeTime", list.get(i).getInvoiceMakeTime());
            lm.put("invoiceUrl", list.get(i).getInvoiceUrl());
            lm.put("enStation", list.get(i).getEnStation());
            lm.put("exStation", list.get(i).getExStation());
            lm.put("exTime", list.get(i).getExTime());
            lm.put("fee", list.get(i).getFee());
            lm.put("totalTaxAmount", list.get(i).getTotalTaxAmount());
            lm.put("plateNum", list.get(i).getPlateNum());
            lm.put("waybillNum", list.get(i).getWaybillNum());
            lm.put("waybillStartTime", list.get(i).getWaybillStartTime());
            lm.put("waybillEndTime", list.get(i).getWaybillEndTime());
            lm.put("totalAmount", list.get(i).getTotalAmount());
            lm.put("taxRate", String.valueOf(list.get(i).getTaxRate()));
            lm.put("amount", list.get(i).getAmount());
            lm.put("sellerName", list.get(i).getSellerName());
            lm.put("sellerTaxpayerCode", list.get(i).getSellerTaxpayerCode());
            lm.put("transactionId", list.get(i).getTransactionId());

            listMap.add(lm);
        }
        map.put("mapList", listMap);

        return ExcelExportUtil.exportExcel(params, map);
    }

    @Override
    public SearchInvoiceResult statisticsData(SearchInvoiceResult searchInvoiceResult, String batchNumber) {
        return baseMapper.statisticsData(searchInvoiceResult, batchNumber);
    }

    @Override
    public List<SearchInvoiceResult> getNumInvoiceByTransactionId(String transactionId) {
        return baseMapper.getNumInvoiceByTransactionId(transactionId);
    }

    @Override
    public List<SearchInvoiceResult> getInvoiceByInvoiceNum(String invoiceNum, String invoiceCode) {
        return baseMapper.getInvoiceByInvoiceNum(invoiceNum, invoiceCode);
    }

    @Override
    public void downloadInvoiceBat(SearchInvoiceResult searchInvoiceResult, String batchNumber, HttpServletResponse response) {
        List<SearchInvoiceResult> searchInvoiceResultList = baseMapper.getInvoiceUrlList(searchInvoiceResult, batchNumber);
        StringBuilder stringBuilder = new StringBuilder();
        stringBuilder.append("echo with wscript:if .arguments.count^<2 then .quit:end if >dl.vbs\n" +
                "echo set aso=.createobject(\"adodb.stream\"):set web=createobject(\"microsoft.xmlhttp\") >>dl.vbs\n" +
                "echo web.open \"get\",.arguments(0),0:web.send:if web.status^>200 then quit >>dl.vbs\n" +
                "echo aso.type=1:aso.open:aso.write web.responsebody:aso.savetofile .arguments(1),2:end with >>dl.vbs");
        stringBuilder.append("\n").append("md invoice");
        for (SearchInvoiceResult invoice : searchInvoiceResultList) {
            stringBuilder.append("\n")
                    .append("cscript dl.vbs ")
                    .append(invoice.getInvoiceUrl())
                    .append(" ")
                    .append("./invoice/")
                    .append(invoice.getInvoiceNum())
                    .append("-")
                    .append(invoice.getInvoiceCode())
                    .append(".pdf");
        }
        stringBuilder.append("\n");
        stringBuilder.append("del dl.vbs");

        InputStream is = null;
        String fileName = System.currentTimeMillis() + ".bat";

        //使用流的形式下载文件
        try {
            //加载文件
            is = new BufferedInputStream(new ByteArrayInputStream(stringBuilder.toString().getBytes("utf-8")));;
            response.addHeader("Content-Disposition", "attachment;filename=" + new String(fileName.getBytes("UTF-8"), "ISO-8859-1"));
            response.addHeader("Content-Length", "" +stringBuilder.toString().length());
            response.setContentType("text/plain");
            response.setContentLengthLong(stringBuilder.toString().length());
            BufferedOutputStream outStream = new BufferedOutputStream(response.getOutputStream());
            byte[] buffer = new byte[1024];
            int bytesRead = 0;
            while ((bytesRead = is.read(buffer)) != -1) {
                outStream.write(buffer, 0, bytesRead);
            }
            outStream.flush();
        } catch (Exception e) {
            log.error("获取附件失败", e);
        }finally {
            try {
                if (null != is) {
                    is.close();
                }
            } catch (IOException ioEx) {
                log.error("", ioEx);
            }

        }
    }
}
