package com.jkcredit.invoice.task;

import com.alibaba.fastjson.JSON;
import com.alibaba.fastjson.JSONObject;
import com.jkcredit.invoice.common.CommonConstants;
import com.jkcredit.invoice.model.entity.CarFreeCarrierBillEnd;
import com.jkcredit.invoice.model.entity.CarFreeCarrierBillStart;
import com.jkcredit.invoice.model.entity.NumInvoice;
import com.jkcredit.invoice.model.entity.User;
import com.jkcredit.invoice.service.CarFreeCarrierBillEndService;
import com.jkcredit.invoice.service.CarFreeCarrierBillStartService;
import com.jkcredit.invoice.service.NumInvoiceService;
import com.jkcredit.invoice.service.UserService;
import com.jkcredit.invoice.util.DateUtil;
import com.jkcredit.invoice.util.HttpUtil;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.scheduling.annotation.Scheduled;
import org.springframework.stereotype.Component;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

/**
 * @description:
 * @author: sunzhaoning
 * @create: 2019-06-27 16:36
 * @version: V1.0
 **/
@Component
@Slf4j
public class ScheduledBillTask {

    @Autowired
    private CarFreeCarrierBillStartService carFreeCarrierBillStartService;

    @Autowired
    private CarFreeCarrierBillEndService carFreeCarrierBillEndService;

    @Autowired
    private UserService userService;

    @Autowired
    private NumInvoiceService numInvoiceService;

    @Autowired
    private HttpUtil httpUtil;

    /**
     * 定时执行已结束运单数据 未修改状态的数据
     */
    @Scheduled(cron = "0 1/5 * * * ?", zone = "Asia/Shanghai")
    public void updateErrorStatus() {
        //查询状态为请求成功，运单状态为未结束运单的数据
        List<CarFreeCarrierBillStart> startList = carFreeCarrierBillStartService.getStartBillByStatus(CommonConstants.UNFINISHED,CommonConstants.SUCCESS,2);
        if(startList.size()>0){
            startList.stream().forEach(startBill ->{
                //查询结束运单中状态为成功的运单
                List<CarFreeCarrierBillEnd> endList = carFreeCarrierBillEndService.getEndBillByNum(startBill.getNum(), CommonConstants.SUCCESS);
                //修改运单状态
                endList.stream().forEach(endBill-> carFreeCarrierBillStartService.updateStatus(endBill.getNum(),CommonConstants.FINISHED));
            });
        }
    }

    /**
     * 定时执行未结束运单 并修改状态的数据
     */
    @Scheduled(cron = "0 1/5 * * * ?", zone = "Asia/Shanghai")
    public void updateUnFinshedBillStatus() {
        //查询状态为请求成功，运单状态为未结束运单的数据
        List<CarFreeCarrierBillStart> startList = carFreeCarrierBillStartService.getStartBillByStatus(CommonConstants.UNFINISHED,CommonConstants.SUCCESS,2);
        if(startList.size()>0){
            startList.stream().forEach(startBill ->{
                String num = startBill.getNum();
                Map<String, Object> endMap = new HashMap<>(6);
                endMap.put("num", num);
                endMap.put("realDestAddr", startBill.getDestAddr());
                endMap.put("endTime", DateUtil.dateToTime(startBill.getPredictEndTime()));

                //调用接口
                String endResult = httpUtil.getCheckResult(endMap, CommonConstants.WAY_BILL_HISTORY_END_API);
                // todo 不能提交！！！
//                String endResult = "{\n" +
//                        "    \"data\": 1,\n" +
//                        "    \"code\": 200,\n" +
//                        "    \"requestid\": \"20190429094207829,503b6b8d-ce1b-4d8f-82a0-1b5b9f6c5792\",\n" +
//                        "    \"msg\": \"{\\\"info\\\":\\\"成功\\\",\\\"receiveTime\\\":\\\"2019-04-29T09:50:16\\\"}\"\n" +
//                        "}";
                log.info("[定时任务]运单结束接口返回：{},请求参数：{}", endResult, JSON.toJSONString(endMap));
                //转换结果为jsonobject
                JSONObject endResultJson = JSON.parseObject(endResult);
                //获取返回data值
                String endDataCode = endResultJson.getString("data");
                //判断返回值是否为1 如果是1 设置状态为成功
                if ("1".equals(endDataCode)) {
                    //修改请求状态为成功
                    carFreeCarrierBillEndService.updateIsSuccess(num,CommonConstants.SUCCESS);
                    //修改运单表状态为一结束
                    carFreeCarrierBillStartService.updateStatus(num,CommonConstants.FINISHED);
                }
            });
        }
    }

    /**
     * 用户设置未开票运单自动开票
     * 实时运单开票（每天23点开始）
     */
    @Scheduled(cron = "0 0 23 * * ?", zone = "Asia/Shanghai")
    public void updateBillInfo() {
        //查询所有用户
        List<User> userList = userService.getAllUser();
        userList.stream().forEach(user -> {
            //查询用户开票中和已结束的实时运单列表
            List<CarFreeCarrierBillStart> carFreeCarrierBillStarts = carFreeCarrierBillStartService.getNotFinishRealTimeList(user.getId());
            if (!carFreeCarrierBillStarts.isEmpty()) {
                //批量查询发票信息
                numInvoiceService.addNumList(carFreeCarrierBillStarts,user.getId());
            }
        });
    }

    /**
     * 用户设置未开票运单自动开票
     * 历史运单开票
     * 每隔一小时
     */
//    @Scheduled(cron = "0 41 15 * * ?", zone = "Asia/Shanghai")
    @Scheduled(cron = "0 0 * * * ?", zone = "Asia/Shanghai")
    public void updateHistoryBillInfo() {
        log.info("[历史运单开票]开始，时间" + System.currentTimeMillis());
        //查询所有用户
        List<User> userList = userService.getAllUser();
        userList.stream().forEach(user -> {
            //查询用户开票中和已结束的历史运单
            List<CarFreeCarrierBillStart> carFreeCarrierBillStarts = carFreeCarrierBillStartService.getNotFinishHistoryList(user.getId());
            if (!carFreeCarrierBillStarts.isEmpty()) {
                //批量查询发票信息
                numInvoiceService.addNumList(carFreeCarrierBillStarts,user.getId());
            }
        });
    }

    /**
     * 预估运单自动开票
     */
//    @Scheduled(cron = "0 10 11 * * ?", zone = "Asia/Shanghai")
    public void autoBillInfo() {
        //获取所有用户
        List<User> userList = userService.getAllUser();
        userList.stream().forEach(user -> {
            //根据用户信息获取用户自动开票运单信息
            List<CarFreeCarrierBillStart> carFreeCarrierBillStarts  = carFreeCarrierBillStartService.getAutoBillList(user.getId());
            List<NumInvoice> numInvoiceList = new ArrayList<>();
            if(carFreeCarrierBillStarts.size()>0){
                carFreeCarrierBillStarts.stream().forEach(carFreeCarrierBillStart -> {
                    NumInvoice numInvoice = new NumInvoice();
                    numInvoice.setNum(carFreeCarrierBillStart.getNum());
                    log.info("[定时任务]预估运单定时开票接口请求参数：{}", carFreeCarrierBillStart.getNum());
                    numInvoiceList.add(numInvoice);
                });
                //调用批量开票接口
//                numInvoiceService.addNumList(numInvoiceList,user.getId());
            }
        });

    }

    /**
     * 定时执行未结束运单 并修改状态的数据
     */
    @Scheduled(cron = "0 1/5 * * * ?", zone = "Asia/Shanghai")
    public void updateAutoUnFinishedBillStatus() {
        //查询状态为请求成功，运单状态为未结束运单的数据
        List<CarFreeCarrierBillStart> startList = carFreeCarrierBillStartService.getStartBillByFlag(CommonConstants.UNFINISHED,CommonConstants.SUCCESS,1,CommonConstants.AUTO_INVOICE);
        if(startList.size()>0){
            startList.stream().forEach(startBill ->{
                String num = startBill.getNum();
                Map<String, Object> endMap = new HashMap<>(6);
                endMap.put("num", num);
                endMap.put("realDestAddr", startBill.getDestAddr());
                endMap.put("endTime", DateUtil.dateToTime(startBill.getPredictEndTime()));

                //调用接口
                String endResult = httpUtil.getCheckResult(endMap, CommonConstants.WAY_BILL_END_API);
                log.info("[定时任务]预估运单结束接口返回：{},请求参数：{}", endResult, JSON.toJSONString(endMap));
                //转换结果为jsonobject
                JSONObject endResultJson = JSON.parseObject(endResult);
                //获取返回data值
                String endDataCode = endResultJson.getString("data");
                //判断返回值是否为1 如果是1 设置状态为成功
                if ("1".equals(endDataCode)) {
                    //修改请求状态为成功
                    carFreeCarrierBillEndService.updateIsSuccess(num,CommonConstants.SUCCESS);
                    //修改运单表状态为一结束
                    carFreeCarrierBillStartService.updateStatus(num,CommonConstants.FINISHED);
                }
            });
        }
    }
}
